@@ -13,15 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import warnings
 from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
+import warnings
 
-from google.api_core import gapic_v1
-from google.api_core import grpc_helpers_async
-from google.api_core import operations_v1
+from google.api_core import gapic_v1, grpc_helpers_async, operations_v1
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
-
+from google.longrunning import operations_pb2  # type: ignore
+from google.protobuf import empty_pb2  # type: ignore
 import grpc  # type: ignore
 from grpc.experimental import aio  # type: ignore
 
@@ -29,9 +28,8 @@ from google.cloud.retail_v2.types import import_config
 from google.cloud.retail_v2.types import product
 from google.cloud.retail_v2.types import product as gcr_product
 from google.cloud.retail_v2.types import product_service
-from google.longrunning import operations_pb2  # type: ignore
-from google.protobuf import empty_pb2  # type: ignore
-from .base import ProductServiceTransport, DEFAULT_CLIENT_INFO
+
+from .base import DEFAULT_CLIENT_INFO, ProductServiceTransport
 from .grpc import ProductServiceGrpcTransport
 
 
@@ -56,7 +54,7 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
     def create_channel(
         cls,
         host: str = "retail.googleapis.com",
-        credentials: ga_credentials.Credentials = None,
+        credentials: Optional[ga_credentials.Credentials] = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -99,17 +97,18 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
         self,
         *,
         host: str = "retail.googleapis.com",
-        credentials: ga_credentials.Credentials = None,
+        credentials: Optional[ga_credentials.Credentials] = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
-        channel: aio.Channel = None,
-        api_mtls_endpoint: str = None,
-        client_cert_source: Callable[[], Tuple[bytes, bytes]] = None,
-        ssl_channel_credentials: grpc.ChannelCredentials = None,
-        client_cert_source_for_mtls: Callable[[], Tuple[bytes, bytes]] = None,
-        quota_project_id=None,
+        channel: Optional[aio.Channel] = None,
+        api_mtls_endpoint: Optional[str] = None,
+        client_cert_source: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
+        ssl_channel_credentials: Optional[grpc.ChannelCredentials] = None,
+        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
+        quota_project_id: Optional[str] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
         always_use_jwt_access: Optional[bool] = False,
+        api_audience: Optional[str] = None,
     ) -> None:
         """Instantiate the transport.
 
@@ -206,6 +205,7 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
             quota_project_id=quota_project_id,
             client_info=client_info,
             always_use_jwt_access=always_use_jwt_access,
+            api_audience=api_audience,
         )
 
         if not self._grpc_channel:
@@ -402,8 +402,8 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
         Bulk import of multiple
         [Product][google.cloud.retail.v2.Product]s.
 
-        Request processing may be synchronous. No partial updating is
-        supported. Non-existing items are created.
+        Request processing may be synchronous. Non-existing items are
+        created.
 
         Note that it is possible for a subset of the
         [Product][google.cloud.retail.v2.Product]s to be successfully
@@ -442,45 +442,56 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
         This process is asynchronous and does not require the
         [Product][google.cloud.retail.v2.Product] to exist before
         updating fulfillment information. If the request is valid, the
-        update will be enqueued and processed downstream. As a
-        consequence, when a response is returned, updates are not
-        immediately manifested in the
-        [Product][google.cloud.retail.v2.Product] queried by
-        [GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
+        update is enqueued and processed downstream. As a consequence,
+        when a response is returned, updates are not immediately
+        manifested in the [Product][google.cloud.retail.v2.Product]
+        queried by
+        [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
         or
-        [ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+        [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
 
         When inventory is updated with
-        [CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
+        [ProductService.CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
         and
-        [UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct],
-        the specified inventory field value(s) will overwrite any
-        existing value(s) while ignoring the last update time for this
-        field. Furthermore, the last update time for the specified
-        inventory fields will be overwritten to the time of the
-        [CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
+        [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct],
+        the specified inventory field value(s) overwrite any existing
+        value(s) while ignoring the last update time for this field.
+        Furthermore, the last update times for the specified inventory
+        fields are overwritten by the times of the
+        [ProductService.CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
         or
-        [UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
+        [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
         request.
 
         If no inventory fields are set in
         [CreateProductRequest.product][google.cloud.retail.v2.CreateProductRequest.product],
-        then any pre-existing inventory information for this product
-        will be used.
+        then any pre-existing inventory information for this product is
+        used.
 
         If no inventory fields are set in
         [SetInventoryRequest.set_mask][google.cloud.retail.v2.SetInventoryRequest.set_mask],
-        then any existing inventory information will be preserved.
+        then any existing inventory information is preserved.
 
         Pre-existing inventory information can only be updated with
-        [SetInventory][google.cloud.retail.v2.ProductService.SetInventory],
-        [AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces],
+        [ProductService.SetInventory][google.cloud.retail.v2.ProductService.SetInventory],
+        [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces],
         and
-        [RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces].
+        [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces].
+
+        The returned [Operation][google.longrunning.Operation]s is
+        obsolete after one day, and the
+        [GetOperation][google.longrunning.Operations.GetOperation] API
+        returns ``NOT_FOUND`` afterwards.
+
+        If conflicting updates are issued, the
+        [Operation][google.longrunning.Operation]s associated with the
+        stale updates are not marked as
+        [done][google.longrunning.Operation.done] until they are
+        obsolete.
 
         This feature is only available for users who have Retail Search
-        enabled. Please enable Retail Search on Cloud Console before
-        using this feature.
+        enabled. Enable Retail Search on Cloud Console before using this
+        feature.
 
         Returns:
             Callable[[~.SetInventoryRequest],
@@ -519,13 +530,23 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
         consequence, when a response is returned, the added place IDs
         are not immediately manifested in the
         [Product][google.cloud.retail.v2.Product] queried by
-        [GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
+        [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
         or
-        [ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+        [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+
+        The returned [Operation][google.longrunning.Operation]s will be
+        obsolete after 1 day, and
+        [GetOperation][google.longrunning.Operations.GetOperation] API
+        will return NOT_FOUND afterwards.
+
+        If conflicting updates are issued, the
+        [Operation][google.longrunning.Operation]s associated with the
+        stale updates will not be marked as
+        [done][google.longrunning.Operation.done] until being obsolete.
 
         This feature is only available for users who have Retail Search
-        enabled. Please enable Retail Search on Cloud Console before
-        using this feature.
+        enabled. Enable Retail Search on Cloud Console before using this
+        feature.
 
         Returns:
             Callable[[~.AddFulfillmentPlacesRequest],
@@ -564,13 +585,23 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
         consequence, when a response is returned, the removed place IDs
         are not immediately manifested in the
         [Product][google.cloud.retail.v2.Product] queried by
-        [GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
+        [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
         or
-        [ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+        [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+
+        The returned [Operation][google.longrunning.Operation]s will be
+        obsolete after 1 day, and
+        [GetOperation][google.longrunning.Operations.GetOperation] API
+        will return NOT_FOUND afterwards.
+
+        If conflicting updates are issued, the
+        [Operation][google.longrunning.Operation]s associated with the
+        stale updates will not be marked as
+        [done][google.longrunning.Operation.done] until being obsolete.
 
         This feature is only available for users who have Retail Search
-        enabled. Please enable Retail Search on Cloud Console before
-        using this feature.
+        enabled. Enable Retail Search on Cloud Console before using this
+        feature.
 
         Returns:
             Callable[[~.RemoveFulfillmentPlacesRequest],
@@ -611,20 +642,30 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
         consequence, when a response is returned, updates are not
         immediately manifested in the
         [Product][google.cloud.retail.v2.Product] queried by
-        [GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
+        [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
         or
-        [ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+        [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
 
         Local inventory information can only be modified using this
         method.
-        [CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
+        [ProductService.CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
         and
-        [UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
+        [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
         has no effect on local inventories.
 
+        The returned [Operation][google.longrunning.Operation]s will be
+        obsolete after 1 day, and
+        [GetOperation][google.longrunning.Operations.GetOperation] API
+        will return NOT_FOUND afterwards.
+
+        If conflicting updates are issued, the
+        [Operation][google.longrunning.Operation]s associated with the
+        stale updates will not be marked as
+        [done][google.longrunning.Operation.done] until being obsolete.
+
         This feature is only available for users who have Retail Search
-        enabled. Please enable Retail Search on Cloud Console before
-        using this feature.
+        enabled. Enable Retail Search on Cloud Console before using this
+        feature.
 
         Returns:
             Callable[[~.AddLocalInventoriesRequest],
@@ -662,20 +703,30 @@ class ProductServiceGrpcAsyncIOTransport(ProductServiceTransport):
         consequence, when a response is returned, removals are not
         immediately manifested in the
         [Product][google.cloud.retail.v2.Product] queried by
-        [GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
+        [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
         or
-        [ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
+        [ProductService.ListProducts][google.cloud.retail.v2.ProductService.ListProducts].
 
         Local inventory information can only be removed using this
         method.
-        [CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
+        [ProductService.CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
         and
-        [UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
+        [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
         has no effect on local inventories.
 
+        The returned [Operation][google.longrunning.Operation]s will be
+        obsolete after 1 day, and
+        [GetOperation][google.longrunning.Operations.GetOperation] API
+        will return NOT_FOUND afterwards.
+
+        If conflicting updates are issued, the
+        [Operation][google.longrunning.Operation]s associated with the
+        stale updates will not be marked as
+        [done][google.longrunning.Operation.done] until being obsolete.
+
         This feature is only available for users who have Retail Search
-        enabled. Please enable Retail Search on Cloud Console before
-        using this feature.
+        enabled. Enable Retail Search on Cloud Console before using this
+        feature.
 
         Returns:
             Callable[[~.RemoveLocalInventoriesRequest],
