@@ -16,19 +16,31 @@
 from collections import OrderedDict
 import os
 import re
-from typing import Dict, Optional, Sequence, Tuple, Type, Union
-import pkg_resources
+from typing import (
+    Dict,
+    Mapping,
+    MutableMapping,
+    MutableSequence,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    Union,
+    cast,
+)
 
 from google.api_core import client_options as client_options_lib
 from google.api_core import exceptions as core_exceptions
 from google.api_core import gapic_v1
 from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
+from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
-from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+from google.cloud.retail_v2 import gapic_version as package_version
+
 try:
     OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
 except AttributeError:  # pragma: NO COVER
@@ -37,14 +49,18 @@ except AttributeError:  # pragma: NO COVER
 from google.api import httpbody_pb2  # type: ignore
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
-from google.cloud.retail_v2.types import common
-from google.cloud.retail_v2.types import import_config
-from google.cloud.retail_v2.types import purge_config
-from google.cloud.retail_v2.types import user_event
-from google.cloud.retail_v2.types import user_event_service
 from google.protobuf import any_pb2  # type: ignore
 from google.protobuf import timestamp_pb2  # type: ignore
-from .transports.base import UserEventServiceTransport, DEFAULT_CLIENT_INFO
+
+from google.cloud.retail_v2.types import (
+    common,
+    import_config,
+    purge_config,
+    user_event,
+    user_event_service,
+)
+
+from .transports.base import DEFAULT_CLIENT_INFO, UserEventServiceTransport
 from .transports.grpc import UserEventServiceGrpcTransport
 from .transports.grpc_asyncio import UserEventServiceGrpcAsyncIOTransport
 
@@ -65,7 +81,7 @@ class UserEventServiceClientMeta(type):
 
     def get_transport_class(
         cls,
-        label: str = None,
+        label: Optional[str] = None,
     ) -> Type[UserEventServiceTransport]:
         """Returns an appropriate transport class.
 
@@ -368,8 +384,8 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         self,
         *,
         credentials: Optional[ga_credentials.Credentials] = None,
-        transport: Union[str, UserEventServiceTransport, None] = None,
-        client_options: Optional[client_options_lib.ClientOptions] = None,
+        transport: Optional[Union[str, UserEventServiceTransport]] = None,
+        client_options: Optional[Union[client_options_lib.ClientOptions, dict]] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
     ) -> None:
         """Instantiates the user event service client.
@@ -383,7 +399,7 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
             transport (Union[str, UserEventServiceTransport]): The
                 transport to use. If set to None, a transport is chosen
                 automatically.
-            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]): Custom options for the
                 client. It won't take effect if a ``transport`` instance is provided.
                 (1) The ``api_endpoint`` property can be used to override the
                 default endpoint provided by the client. GOOGLE_API_USE_MTLS_ENDPOINT
@@ -413,6 +429,7 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
             client_options = client_options_lib.from_dict(client_options)
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
+        client_options = cast(client_options_lib.ClientOptions, client_options)
 
         api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
             client_options
@@ -460,20 +477,28 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
                 quota_project_id=client_options.quota_project_id,
                 client_info=client_info,
                 always_use_jwt_access=True,
+                api_audience=client_options.api_audience,
             )
 
     def write_user_event(
         self,
-        request: Union[user_event_service.WriteUserEventRequest, dict] = None,
+        request: Optional[Union[user_event_service.WriteUserEventRequest, dict]] = None,
         *,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
-        timeout: float = None,
+        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> user_event.UserEvent:
         r"""Writes a single user event.
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import retail_v2
 
             def sample_write_user_event():
@@ -545,10 +570,12 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
 
     def collect_user_event(
         self,
-        request: Union[user_event_service.CollectUserEventRequest, dict] = None,
+        request: Optional[
+            Union[user_event_service.CollectUserEventRequest, dict]
+        ] = None,
         *,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
-        timeout: float = None,
+        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> httpbody_pb2.HttpBody:
         r"""Writes a single user event from the browser. This
@@ -558,9 +585,15 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         pixel and Google Tag Manager. Users should not call this
         method directly.
 
-
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import retail_v2
 
             def sample_collect_user_event():
@@ -569,6 +602,7 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
 
                 # Initialize request argument(s)
                 request = retail_v2.CollectUserEventRequest(
+                    prebuilt_rule="prebuilt_rule_value",
                     parent="parent_value",
                     user_event="user_event_value",
                 )
@@ -670,10 +704,10 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
 
     def purge_user_events(
         self,
-        request: Union[purge_config.PurgeUserEventsRequest, dict] = None,
+        request: Optional[Union[purge_config.PurgeUserEventsRequest, dict]] = None,
         *,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
-        timeout: float = None,
+        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
         r"""Deletes permanently all user events specified by the
@@ -682,9 +716,15 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         or days to complete. To test a filter, use the list
         command first.
 
-
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import retail_v2
 
             def sample_purge_user_events():
@@ -765,23 +805,29 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
 
     def import_user_events(
         self,
-        request: Union[import_config.ImportUserEventsRequest, dict] = None,
+        request: Optional[Union[import_config.ImportUserEventsRequest, dict]] = None,
         *,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
-        timeout: float = None,
+        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
-        r"""Bulk import of User events. Request processing might
-        be synchronous. Events that already exist are skipped.
-        Use this method for backfilling historical user events.
-        Operation.response is of type ImportResponse. Note that
-        it is possible for a subset of the items to be
-        successfully inserted. Operation.metadata is of type
-        ImportMetadata.
+        r"""Bulk import of User events. Request processing might be
+        synchronous. Events that already exist are skipped. Use this
+        method for backfilling historical user events.
 
+        ``Operation.response`` is of type ``ImportResponse``. Note that
+        it is possible for a subset of the items to be successfully
+        inserted. ``Operation.metadata`` is of type ``ImportMetadata``.
 
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import retail_v2
 
             def sample_import_user_events():
@@ -868,27 +914,35 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
 
     def rejoin_user_events(
         self,
-        request: Union[user_event_service.RejoinUserEventsRequest, dict] = None,
+        request: Optional[
+            Union[user_event_service.RejoinUserEventsRequest, dict]
+        ] = None,
         *,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
-        timeout: float = None,
+        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
-        r"""Starts a user event rejoin operation with latest
-        product catalog. Events will not be annotated with
-        detailed product information if product is missing from
-        the catalog at the time the user event is ingested, and
-        these events are stored as unjoined events with a
-        limited usage on training and serving. This method can
-        be used to start a join operation on specified events
-        with latest version of product catalog. It can also be
-        used to correct events joined with the wrong product
+        r"""Starts a user-event rejoin operation with latest
+        product catalog. Events are not annotated with detailed
+        product information for products that are missing from
+        the catalog when the user event is ingested. These
+        events are stored as unjoined events with limited usage
+        on training and serving. You can use this method to
+        start a join operation on specified events with the
+        latest version of product catalog. You can also use this
+        method to correct events joined with the wrong product
         catalog. A rejoin operation can take hours or days to
         complete.
 
-
         .. code-block:: python
 
+            # This snippet has been automatically generated and should be regarded as a
+            # code template only.
+            # It will require modifications to work:
+            # - It may require correct/in-range values for request initialization.
+            # - It may require specifying regional endpoints when creating the service
+            #   client as shown in:
+            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
             from google.cloud import retail_v2
 
             def sample_rejoin_user_events():
@@ -980,14 +1034,9 @@ class UserEventServiceClient(metaclass=UserEventServiceClientMeta):
         self.transport.close()
 
 
-try:
-    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
-        gapic_version=pkg_resources.get_distribution(
-            "google-cloud-retail",
-        ).version,
-    )
-except pkg_resources.DistributionNotFound:
-    DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()
+DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
+    gapic_version=package_version.__version__
+)
 
 
 __all__ = ("UserEventServiceClient",)
