@@ -13,15 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import proto  # type: ignore
+from typing import MutableMapping, MutableSequence
 
-from google.cloud.retail_v2.types import product
-from google.cloud.retail_v2.types import user_event
 from google.protobuf import field_mask_pb2  # type: ignore
 from google.protobuf import timestamp_pb2  # type: ignore
 from google.rpc import status_pb2  # type: ignore
 from google.type import date_pb2  # type: ignore
+import proto  # type: ignore
 
+from google.cloud.retail_v2.types import product, user_event
 
 __protobuf__ = proto.module(
     package="google.cloud.retail.v2",
@@ -48,10 +48,9 @@ __protobuf__ = proto.module(
 
 class GcsSource(proto.Message):
     r"""Google Cloud Storage location for input content.
-    format.
 
     Attributes:
-        input_uris (Sequence[str]):
+        input_uris (MutableSequence[str]):
             Required. Google Cloud Storage URIs to input files. URI can
             be up to 2000 characters long. URIs can match the full
             object path (for example,
@@ -83,19 +82,21 @@ class GcsSource(proto.Message):
 
             Supported values for control imports:
 
-            -  'control' (default): One JSON [Control][] per line.
+            -  ``control`` (default): One JSON
+               [Control][google.cloud.retail.v2.Control] per line.
 
             Supported values for catalog attribute imports:
 
-            -  'catalog_attribute' (default): One CSV
-               [CatalogAttribute][] per line.
+            -  ``catalog_attribute`` (default): One CSV
+               [CatalogAttribute][google.cloud.retail.v2.CatalogAttribute]
+               per line.
     """
 
-    input_uris = proto.RepeatedField(
+    input_uris: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=1,
     )
-    data_schema = proto.Field(
+    data_schema: str = proto.Field(
         proto.STRING,
         number=2,
     )
@@ -111,9 +112,8 @@ class BigQuerySource(proto.Message):
             BigQuery time partitioned table's \_PARTITIONDATE in
             YYYY-MM-DD format.
 
-            Only supported when
-            [ImportProductsRequest.reconciliation_mode][google.cloud.retail.v2.ImportProductsRequest.reconciliation_mode]
-            is set to ``FULL``.
+            Only supported in
+            [ImportProductsRequest][google.cloud.retail.v2.ImportProductsRequest].
 
             This field is a member of `oneof`_ ``partition``.
         project_id (str):
@@ -153,9 +153,7 @@ class BigQuerySource(proto.Message):
                [UserEvent][google.cloud.retail.v2.UserEvent] per line.
             -  ``user_event_ga360``: The schema is available here:
                https://support.google.com/analytics/answer/3437719.
-            -  ``user_event_ga4``: This feature is in private preview.
-               Please contact the support team for importing Google
-               Analytics 4 events. The schema is available here:
+            -  ``user_event_ga4``: The schema is available here:
                https://support.google.com/analytics/answer/7029846.
 
             Supported values for auto-completion imports:
@@ -166,29 +164,29 @@ class BigQuerySource(proto.Message):
             -  ``allowlist``: One JSON allow suggestion per line.
     """
 
-    partition_date = proto.Field(
+    partition_date: date_pb2.Date = proto.Field(
         proto.MESSAGE,
         number=6,
         oneof="partition",
         message=date_pb2.Date,
     )
-    project_id = proto.Field(
+    project_id: str = proto.Field(
         proto.STRING,
         number=5,
     )
-    dataset_id = proto.Field(
+    dataset_id: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    table_id = proto.Field(
+    table_id: str = proto.Field(
         proto.STRING,
         number=2,
     )
-    gcs_staging_dir = proto.Field(
+    gcs_staging_dir: str = proto.Field(
         proto.STRING,
         number=3,
     )
-    data_schema = proto.Field(
+    data_schema: str = proto.Field(
         proto.STRING,
         number=4,
     )
@@ -199,14 +197,14 @@ class ProductInlineSource(proto.Message):
     method.
 
     Attributes:
-        products (Sequence[google.cloud.retail_v2.types.Product]):
+        products (MutableSequence[google.cloud.retail_v2.types.Product]):
             Required. A list of products to update/create. Each product
             must have a valid
             [Product.id][google.cloud.retail.v2.Product.id]. Recommended
             max of 100 items.
     """
 
-    products = proto.RepeatedField(
+    products: MutableSequence[product.Product] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=product.Product,
@@ -218,12 +216,12 @@ class UserEventInlineSource(proto.Message):
     method.
 
     Attributes:
-        user_events (Sequence[google.cloud.retail_v2.types.UserEvent]):
+        user_events (MutableSequence[google.cloud.retail_v2.types.UserEvent]):
             Required. A list of user events to import.
             Recommended max of 10k items.
     """
 
-    user_events = proto.RepeatedField(
+    user_events: MutableSequence[user_event.UserEvent] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=user_event.UserEvent,
@@ -239,13 +237,13 @@ class ImportErrorsConfig(proto.Message):
         gcs_prefix (str):
             Google Cloud Storage prefix for import errors. This must be
             an empty, existing Cloud Storage directory. Import errors
-            will be written to sharded files in this directory, one per
+            are written to sharded files in this directory, one per
             line, as a JSON-encoded ``google.rpc.Status`` message.
 
             This field is a member of `oneof`_ ``destination``.
     """
 
-    gcs_prefix = proto.Field(
+    gcs_prefix: str = proto.Field(
         proto.STRING,
         number=1,
         oneof="destination",
@@ -272,24 +270,26 @@ class ImportProductsRequest(proto.Message):
             The desired location of errors incurred
             during the Import.
         update_mask (google.protobuf.field_mask_pb2.FieldMask):
-            Indicates which fields in the provided
-            imported 'products' to update. If not set, will
-            by default update all fields.
+            Indicates which fields in the provided imported ``products``
+            to update. If not set, all fields are updated.
         reconciliation_mode (google.cloud.retail_v2.types.ImportProductsRequest.ReconciliationMode):
             The mode of reconciliation between existing products and the
             products to be imported. Defaults to
             [ReconciliationMode.INCREMENTAL][google.cloud.retail.v2.ImportProductsRequest.ReconciliationMode.INCREMENTAL].
         notification_pubsub_topic (str):
-            Pub/Sub topic for receiving notification. If this field is
-            set, when the import is finished, a notification will be
-            sent to specified Pub/Sub topic. The message data will be
-            JSON string of a [Operation][google.longrunning.Operation].
-            Format of the Pub/Sub topic is
-            ``projects/{project}/topics/{topic}``.
+            Full Pub/Sub topic name for receiving notification. If this
+            field is set, when the import is finished, a notification is
+            sent to specified Pub/Sub topic. The message data is JSON
+            string of a [Operation][google.longrunning.Operation].
 
-            Only supported when
-            [ImportProductsRequest.reconciliation_mode][google.cloud.retail.v2.ImportProductsRequest.reconciliation_mode]
-            is set to ``FULL``.
+            Format of the Pub/Sub topic is
+            ``projects/{project}/topics/{topic}``. It has to be within
+            the same project as
+            [ImportProductsRequest.parent][google.cloud.retail.v2.ImportProductsRequest.parent].
+            Make sure that
+            ``service-<project number>@gcp-sa-retail.iam.gserviceaccount.com``
+            has the ``pubsub.topics.publish`` IAM permission on the
+            topic.
     """
 
     class ReconciliationMode(proto.Enum):
@@ -300,35 +300,35 @@ class ImportProductsRequest(proto.Message):
         INCREMENTAL = 1
         FULL = 2
 
-    parent = proto.Field(
+    parent: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    request_id = proto.Field(
+    request_id: str = proto.Field(
         proto.STRING,
         number=6,
     )
-    input_config = proto.Field(
+    input_config: "ProductInputConfig" = proto.Field(
         proto.MESSAGE,
         number=2,
         message="ProductInputConfig",
     )
-    errors_config = proto.Field(
+    errors_config: "ImportErrorsConfig" = proto.Field(
         proto.MESSAGE,
         number=3,
         message="ImportErrorsConfig",
     )
-    update_mask = proto.Field(
+    update_mask: field_mask_pb2.FieldMask = proto.Field(
         proto.MESSAGE,
         number=4,
         message=field_mask_pb2.FieldMask,
     )
-    reconciliation_mode = proto.Field(
+    reconciliation_mode: ReconciliationMode = proto.Field(
         proto.ENUM,
         number=5,
         enum=ReconciliationMode,
     )
-    notification_pubsub_topic = proto.Field(
+    notification_pubsub_topic: str = proto.Field(
         proto.STRING,
         number=7,
     )
@@ -350,16 +350,16 @@ class ImportUserEventsRequest(proto.Message):
             event imports.
     """
 
-    parent = proto.Field(
+    parent: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    input_config = proto.Field(
+    input_config: "UserEventInputConfig" = proto.Field(
         proto.MESSAGE,
         number=2,
         message="UserEventInputConfig",
     )
-    errors_config = proto.Field(
+    errors_config: "ImportErrorsConfig" = proto.Field(
         proto.MESSAGE,
         number=3,
         message="ImportErrorsConfig",
@@ -381,23 +381,22 @@ class ImportCompletionDataRequest(proto.Message):
             data.
         notification_pubsub_topic (str):
             Pub/Sub topic for receiving notification. If this field is
-            set, when the import is finished, a notification will be
-            sent to specified Pub/Sub topic. The message data will be
-            JSON string of a [Operation][google.longrunning.Operation].
-            Format of the Pub/Sub topic is
-            ``projects/{project}/topics/{topic}``.
+            set, when the import is finished, a notification is sent to
+            specified Pub/Sub topic. The message data is JSON string of
+            a [Operation][google.longrunning.Operation]. Format of the
+            Pub/Sub topic is ``projects/{project}/topics/{topic}``.
     """
 
-    parent = proto.Field(
+    parent: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    input_config = proto.Field(
+    input_config: "CompletionDataInputConfig" = proto.Field(
         proto.MESSAGE,
         number=2,
         message="CompletionDataInputConfig",
     )
-    notification_pubsub_topic = proto.Field(
+    notification_pubsub_topic: str = proto.Field(
         proto.STRING,
         number=3,
     )
@@ -430,19 +429,19 @@ class ProductInputConfig(proto.Message):
             This field is a member of `oneof`_ ``source``.
     """
 
-    product_inline_source = proto.Field(
+    product_inline_source: "ProductInlineSource" = proto.Field(
         proto.MESSAGE,
         number=1,
         oneof="source",
         message="ProductInlineSource",
     )
-    gcs_source = proto.Field(
+    gcs_source: "GcsSource" = proto.Field(
         proto.MESSAGE,
         number=2,
         oneof="source",
         message="GcsSource",
     )
-    big_query_source = proto.Field(
+    big_query_source: "BigQuerySource" = proto.Field(
         proto.MESSAGE,
         number=3,
         oneof="source",
@@ -477,19 +476,19 @@ class UserEventInputConfig(proto.Message):
             This field is a member of `oneof`_ ``source``.
     """
 
-    user_event_inline_source = proto.Field(
+    user_event_inline_source: "UserEventInlineSource" = proto.Field(
         proto.MESSAGE,
         number=1,
         oneof="source",
         message="UserEventInlineSource",
     )
-    gcs_source = proto.Field(
+    gcs_source: "GcsSource" = proto.Field(
         proto.MESSAGE,
         number=2,
         oneof="source",
         message="GcsSource",
     )
-    big_query_source = proto.Field(
+    big_query_source: "BigQuerySource" = proto.Field(
         proto.MESSAGE,
         number=3,
         oneof="source",
@@ -514,7 +513,7 @@ class CompletionDataInputConfig(proto.Message):
             This field is a member of `oneof`_ ``source``.
     """
 
-    big_query_source = proto.Field(
+    big_query_source: "BigQuerySource" = proto.Field(
         proto.MESSAGE,
         number=1,
         oneof="source",
@@ -524,8 +523,8 @@ class CompletionDataInputConfig(proto.Message):
 
 class ImportMetadata(proto.Message):
     r"""Metadata related to the progress of the Import operation.
-    This will be returned by the
-    google.longrunning.Operation.metadata field.
+    This is returned by the google.longrunning.Operation.metadata
+    field.
 
     Attributes:
         create_time (google.protobuf.timestamp_pb2.Timestamp):
@@ -543,36 +542,35 @@ class ImportMetadata(proto.Message):
             Deprecated. This field is never set.
         notification_pubsub_topic (str):
             Pub/Sub topic for receiving notification. If this field is
-            set, when the import is finished, a notification will be
-            sent to specified Pub/Sub topic. The message data will be
-            JSON string of a [Operation][google.longrunning.Operation].
-            Format of the Pub/Sub topic is
-            ``projects/{project}/topics/{topic}``.
+            set, when the import is finished, a notification is sent to
+            specified Pub/Sub topic. The message data is JSON string of
+            a [Operation][google.longrunning.Operation]. Format of the
+            Pub/Sub topic is ``projects/{project}/topics/{topic}``.
     """
 
-    create_time = proto.Field(
+    create_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=1,
         message=timestamp_pb2.Timestamp,
     )
-    update_time = proto.Field(
+    update_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=2,
         message=timestamp_pb2.Timestamp,
     )
-    success_count = proto.Field(
+    success_count: int = proto.Field(
         proto.INT64,
         number=3,
     )
-    failure_count = proto.Field(
+    failure_count: int = proto.Field(
         proto.INT64,
         number=4,
     )
-    request_id = proto.Field(
+    request_id: str = proto.Field(
         proto.STRING,
         number=5,
     )
-    notification_pubsub_topic = proto.Field(
+    notification_pubsub_topic: str = proto.Field(
         proto.STRING,
         number=6,
     )
@@ -586,7 +584,7 @@ class ImportProductsResponse(proto.Message):
     was successful.
 
     Attributes:
-        error_samples (Sequence[google.rpc.status_pb2.Status]):
+        error_samples (MutableSequence[google.rpc.status_pb2.Status]):
             A sample of errors encountered while
             processing the request.
         errors_config (google.cloud.retail_v2.types.ImportErrorsConfig):
@@ -594,12 +592,12 @@ class ImportProductsResponse(proto.Message):
             errors in the request if set.
     """
 
-    error_samples = proto.RepeatedField(
+    error_samples: MutableSequence[status_pb2.Status] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=status_pb2.Status,
     )
-    errors_config = proto.Field(
+    errors_config: "ImportErrorsConfig" = proto.Field(
         proto.MESSAGE,
         number=2,
         message="ImportErrorsConfig",
@@ -613,7 +611,7 @@ class ImportUserEventsResponse(proto.Message):
     was successful.
 
     Attributes:
-        error_samples (Sequence[google.rpc.status_pb2.Status]):
+        error_samples (MutableSequence[google.rpc.status_pb2.Status]):
             A sample of errors encountered while
             processing the request.
         errors_config (google.cloud.retail_v2.types.ImportErrorsConfig):
@@ -624,17 +622,17 @@ class ImportUserEventsResponse(proto.Message):
             status.
     """
 
-    error_samples = proto.RepeatedField(
+    error_samples: MutableSequence[status_pb2.Status] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=status_pb2.Status,
     )
-    errors_config = proto.Field(
+    errors_config: "ImportErrorsConfig" = proto.Field(
         proto.MESSAGE,
         number=2,
         message="ImportErrorsConfig",
     )
-    import_summary = proto.Field(
+    import_summary: "UserEventImportSummary" = proto.Field(
         proto.MESSAGE,
         number=3,
         message="UserEventImportSummary",
@@ -655,11 +653,11 @@ class UserEventImportSummary(proto.Message):
             catalog.
     """
 
-    joined_events_count = proto.Field(
+    joined_events_count: int = proto.Field(
         proto.INT64,
         number=1,
     )
-    unjoined_events_count = proto.Field(
+    unjoined_events_count: int = proto.Field(
         proto.INT64,
         number=2,
     )
@@ -673,12 +671,12 @@ class ImportCompletionDataResponse(proto.Message):
     successful.
 
     Attributes:
-        error_samples (Sequence[google.rpc.status_pb2.Status]):
+        error_samples (MutableSequence[google.rpc.status_pb2.Status]):
             A sample of errors encountered while
             processing the request.
     """
 
-    error_samples = proto.RepeatedField(
+    error_samples: MutableSequence[status_pb2.Status] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=status_pb2.Status,
