@@ -13,11 +13,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+from typing import MutableMapping, MutableSequence
+
+from google.protobuf import struct_pb2  # type: ignore
 import proto  # type: ignore
 
 from google.cloud.retail_v2.types import user_event as gcr_user_event
-from google.protobuf import struct_pb2  # type: ignore
-
 
 __protobuf__ = proto.module(
     package="google.cloud.retail.v2",
@@ -34,30 +35,46 @@ class PredictRequest(proto.Message):
     Attributes:
         placement (str):
             Required. Full resource name of the format:
-            {name=projects/*/locations/global/catalogs/default_catalog/placements/*}
-            The ID of the Recommendations AI placement. Before you can
-            request predictions from your model, you must create at
-            least one placement for it. For more information, see
-            `Managing
-            placements <https://cloud.google.com/retail/recommendations-ai/docs/manage-placements>`__.
+            ``{placement=projects/*/locations/global/catalogs/default_catalog/servingConfigs/*}``
+            or
+            ``{placement=projects/*/locations/global/catalogs/default_catalog/placements/*}``.
+            We recommend using the ``servingConfigs`` resource.
+            ``placements`` is a legacy resource. The ID of the
+            Recommendations AI serving config or placement. Before you
+            can request predictions from your model, you must create at
+            least one serving config or placement for it. For more
+            information, see [Manage serving configs]
+            (https://cloud.google.com/retail/docs/manage-configs).
 
-            The full list of available placements can be seen at
-            https://console.cloud.google.com/recommendation/catalogs/default_catalog/placements
+            The full list of available serving configs can be seen at
+            https://console.cloud.google.com/ai/retail/catalogs/default_catalog/configs
         user_event (google.cloud.retail_v2.types.UserEvent):
-            Required. Context about the user, what they
-            are looking at and what action they took to
-            trigger the predict request. Note that this user
-            event detail won't be ingested to userEvent
-            logs. Thus, a separate userEvent write request
-            is required for event logging.
+            Required. Context about the user, what they are looking at
+            and what action they took to trigger the predict request.
+            Note that this user event detail won't be ingested to
+            userEvent logs. Thus, a separate userEvent write request is
+            required for event logging.
+
+            Don't set
+            [UserEvent.visitor_id][google.cloud.retail.v2.UserEvent.visitor_id]
+            or
+            [UserInfo.user_id][google.cloud.retail.v2.UserInfo.user_id]
+            to the same fixed ID for different users. If you are trying
+            to receive non-personalized recommendations (not
+            recommended; this can negatively impact model performance),
+            instead set
+            [UserEvent.visitor_id][google.cloud.retail.v2.UserEvent.visitor_id]
+            to a random unique ID and leave
+            [UserInfo.user_id][google.cloud.retail.v2.UserInfo.user_id]
+            unset.
         page_size (int):
-            Maximum number of results to return per page.
-            Set this property to the number of prediction
-            results needed. If zero, the service will choose
-            a reasonable default. The maximum allowed value
-            is 100. Values above 100 will be coerced to 100.
+            Maximum number of results to return. Set this
+            property to the number of prediction results
+            needed. If zero, the service will choose a
+            reasonable default. The maximum allowed value is
+            100. Values above 100 will be coerced to 100.
         page_token (str):
-            The previous PredictResponse.next_page_token.
+            This field is not used; leave it unset.
         filter (str):
             Filter for restricting prediction results with a length
             limit of 5,000 characters. Accepts values for tags and the
@@ -86,19 +103,27 @@ class PredictRequest(proto.Message):
             -  filterOutOfStockItems
 
             If your filter blocks all prediction results, the API will
-            return generic (unfiltered) popular products. If you only
-            want results strictly matching the filters, set
-            ``strictFiltering`` to True in ``PredictRequest.params`` to
-            receive empty results instead. Note that the API will never
-            return items with storageStatus of "EXPIRED" or "DELETED"
-            regardless of filter choices.
+            return *no* results. If instead you want empty result sets
+            to return generic (unfiltered) popular products, set
+            ``strictFiltering`` to False in ``PredictRequest.params``.
+            Note that the API will never return items with storageStatus
+            of "EXPIRED" or "DELETED" regardless of filter choices.
+
+            If ``filterSyntaxV2`` is set to true under the ``params``
+            field, then attribute-based expressions are expected instead
+            of the above described tag-based syntax. Examples:
+
+            -  (colors: ANY("Red", "Blue")) AND NOT (categories:
+               ANY("Phones"))
+            -  (availability: ANY("IN_STOCK")) AND (colors: ANY("Red")
+               OR categories: ANY("Phones"))
         validate_only (bool):
             Use validate only mode for this prediction
             query. If set to true, a dummy model will be
             used that returns arbitrary products. Note that
             the validate only mode should only be used for
             testing the API, or if the model is not ready.
-        params (Sequence[google.cloud.retail_v2.types.PredictRequest.ParamsEntry]):
+        params (MutableMapping[str, google.protobuf.struct_pb2.Value]):
             Additional domain specific parameters for the predictions.
 
             Allowed values:
@@ -110,7 +135,7 @@ class PredictRequest(proto.Message):
                'score' corresponding to each returned product will be
                set in the ``results.metadata`` field in the prediction
                response. The given 'score' indicates the probability of
-               an product being clicked/purchased given the user's
+               a product being clicked/purchased given the user's
                context and history.
             -  ``strictFiltering``: Boolean. True by default. If set to
                false, the service will return generic (unfiltered)
@@ -127,7 +152,10 @@ class PredictRequest(proto.Message):
                'low-diversity', 'medium-diversity', 'high-diversity',
                'auto-diversity'}. This gives request-level control and
                adjusts prediction results based on product category.
-        labels (Sequence[google.cloud.retail_v2.types.PredictRequest.LabelsEntry]):
+            -  ``filterSyntaxV2``: Boolean. False by default. If set to
+               true, the ``filter`` field is interpreteted according to
+               the new, attribute-based syntax.
+        labels (MutableMapping[str, str]):
             The labels applied to a resource must meet the following
             requirements:
 
@@ -135,8 +163,8 @@ class PredictRequest(proto.Message):
                of 64.
             -  Each label must be a key-value pair.
             -  Keys have a minimum length of 1 character and a maximum
-               length of 63 characters, and cannot be empty. Values can
-               be empty, and have a maximum length of 63 characters.
+               length of 63 characters and cannot be empty. Values can
+               be empty and have a maximum length of 63 characters.
             -  Keys and values can contain only lowercase letters,
                numeric characters, underscores, and dashes. All
                characters must use UTF-8 encoding, and international
@@ -151,38 +179,38 @@ class PredictRequest(proto.Message):
             for more details.
     """
 
-    placement = proto.Field(
+    placement: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    user_event = proto.Field(
+    user_event: gcr_user_event.UserEvent = proto.Field(
         proto.MESSAGE,
         number=2,
         message=gcr_user_event.UserEvent,
     )
-    page_size = proto.Field(
+    page_size: int = proto.Field(
         proto.INT32,
         number=3,
     )
-    page_token = proto.Field(
+    page_token: str = proto.Field(
         proto.STRING,
         number=4,
     )
-    filter = proto.Field(
+    filter: str = proto.Field(
         proto.STRING,
         number=5,
     )
-    validate_only = proto.Field(
+    validate_only: bool = proto.Field(
         proto.BOOL,
         number=6,
     )
-    params = proto.MapField(
+    params: MutableMapping[str, struct_pb2.Value] = proto.MapField(
         proto.STRING,
         proto.MESSAGE,
         number=7,
         message=struct_pb2.Value,
     )
-    labels = proto.MapField(
+    labels: MutableMapping[str, str] = proto.MapField(
         proto.STRING,
         proto.STRING,
         number=8,
@@ -193,7 +221,7 @@ class PredictResponse(proto.Message):
     r"""Response message for predict method.
 
     Attributes:
-        results (Sequence[google.cloud.retail_v2.types.PredictResponse.PredictionResult]):
+        results (MutableSequence[google.cloud.retail_v2.types.PredictResponse.PredictionResult]):
             A list of recommended products. The order
             represents the ranking (from the most relevant
             product to the least).
@@ -202,7 +230,7 @@ class PredictResponse(proto.Message):
             [UserEvent][google.cloud.retail.v2.UserEvent] logs resulting
             from this recommendation, which enables accurate attribution
             of recommendation model performance.
-        missing_ids (Sequence[str]):
+        missing_ids (MutableSequence[str]):
             IDs of products in the request that were
             missing from the inventory.
         validate_only (bool):
@@ -217,44 +245,44 @@ class PredictResponse(proto.Message):
         Attributes:
             id (str):
                 ID of the recommended product
-            metadata (Sequence[google.cloud.retail_v2.types.PredictResponse.PredictionResult.MetadataEntry]):
+            metadata (MutableMapping[str, google.protobuf.struct_pb2.Value]):
                 Additional product metadata / annotations.
 
                 Possible values:
 
-                -  ``product``: JSON representation of the product. Will be
-                   set if ``returnProduct`` is set to true in
+                -  ``product``: JSON representation of the product. Is set
+                   if ``returnProduct`` is set to true in
                    ``PredictRequest.params``.
-                -  ``score``: Prediction score in double value. Will be set
-                   if ``returnScore`` is set to true in
+                -  ``score``: Prediction score in double value. Is set if
+                   ``returnScore`` is set to true in
                    ``PredictRequest.params``.
         """
 
-        id = proto.Field(
+        id: str = proto.Field(
             proto.STRING,
             number=1,
         )
-        metadata = proto.MapField(
+        metadata: MutableMapping[str, struct_pb2.Value] = proto.MapField(
             proto.STRING,
             proto.MESSAGE,
             number=2,
             message=struct_pb2.Value,
         )
 
-    results = proto.RepeatedField(
+    results: MutableSequence[PredictionResult] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=PredictionResult,
     )
-    attribution_token = proto.Field(
+    attribution_token: str = proto.Field(
         proto.STRING,
         number=2,
     )
-    missing_ids = proto.RepeatedField(
+    missing_ids: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=3,
     )
-    validate_only = proto.Field(
+    validate_only: bool = proto.Field(
         proto.BOOL,
         number=4,
     )
