@@ -13,13 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+from typing import MutableMapping, MutableSequence
+
+from google.protobuf import field_mask_pb2  # type: ignore
+from google.protobuf import timestamp_pb2  # type: ignore
 import proto  # type: ignore
 
 from google.cloud.retail_v2.types import common
 from google.cloud.retail_v2.types import product as gcr_product
-from google.protobuf import field_mask_pb2  # type: ignore
-from google.protobuf import timestamp_pb2  # type: ignore
-
 
 __protobuf__ = proto.module(
     package="google.cloud.retail.v2",
@@ -50,7 +51,9 @@ __protobuf__ = proto.module(
 
 
 class CreateProductRequest(proto.Message):
-    r"""Request message for [CreateProduct][] method.
+    r"""Request message for
+    [ProductService.CreateProduct][google.cloud.retail.v2.ProductService.CreateProduct]
+    method.
 
     Attributes:
         parent (str):
@@ -80,23 +83,25 @@ class CreateProductRequest(proto.Message):
             error is returned.
     """
 
-    parent = proto.Field(
+    parent: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    product = proto.Field(
+    product: gcr_product.Product = proto.Field(
         proto.MESSAGE,
         number=2,
         message=gcr_product.Product,
     )
-    product_id = proto.Field(
+    product_id: str = proto.Field(
         proto.STRING,
         number=3,
     )
 
 
 class GetProductRequest(proto.Message):
-    r"""Request message for [GetProduct][] method.
+    r"""Request message for
+    [ProductService.GetProduct][google.cloud.retail.v2.ProductService.GetProduct]
+    method.
 
     Attributes:
         name (str):
@@ -113,14 +118,16 @@ class GetProductRequest(proto.Message):
             does not exist, a NOT_FOUND error is returned.
     """
 
-    name = proto.Field(
+    name: str = proto.Field(
         proto.STRING,
         number=1,
     )
 
 
 class UpdateProductRequest(proto.Message):
-    r"""Request message for [UpdateProduct][] method.
+    r"""Request message for
+    [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
+    method.
 
     Attributes:
         product (google.cloud.retail_v2.types.Product):
@@ -144,6 +151,11 @@ class UpdateProductRequest(proto.Message):
 
             If an unsupported or unknown field is provided, an
             INVALID_ARGUMENT error is returned.
+
+            The attribute key can be updated by setting the mask path as
+            "attributes.${key_name}". If a key name is present in the
+            mask but not in the patching product from the request, this
+            key will be deleted after the update.
         allow_missing (bool):
             If set to true, and the
             [Product][google.cloud.retail.v2.Product] is not found, a
@@ -151,24 +163,26 @@ class UpdateProductRequest(proto.Message):
             created. In this situation, ``update_mask`` is ignored.
     """
 
-    product = proto.Field(
+    product: gcr_product.Product = proto.Field(
         proto.MESSAGE,
         number=1,
         message=gcr_product.Product,
     )
-    update_mask = proto.Field(
+    update_mask: field_mask_pb2.FieldMask = proto.Field(
         proto.MESSAGE,
         number=2,
         message=field_mask_pb2.FieldMask,
     )
-    allow_missing = proto.Field(
+    allow_missing: bool = proto.Field(
         proto.BOOL,
         number=3,
     )
 
 
 class DeleteProductRequest(proto.Message):
-    r"""Request message for [DeleteProduct][] method.
+    r"""Request message for
+    [ProductService.DeleteProduct][google.cloud.retail.v2.ProductService.DeleteProduct]
+    method.
 
     Attributes:
         name (str):
@@ -197,7 +211,7 @@ class DeleteProductRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product] will be deleted.
     """
 
-    name = proto.Field(
+    name: str = proto.Field(
         proto.STRING,
         number=1,
     )
@@ -290,23 +304,23 @@ class ListProductsRequest(proto.Message):
             INVALID_ARGUMENT error is returned.
     """
 
-    parent = proto.Field(
+    parent: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    page_size = proto.Field(
+    page_size: int = proto.Field(
         proto.INT32,
         number=2,
     )
-    page_token = proto.Field(
+    page_token: str = proto.Field(
         proto.STRING,
         number=3,
     )
-    filter = proto.Field(
+    filter: str = proto.Field(
         proto.STRING,
         number=4,
     )
-    read_mask = proto.Field(
+    read_mask: field_mask_pb2.FieldMask = proto.Field(
         proto.MESSAGE,
         number=5,
         message=field_mask_pb2.FieldMask,
@@ -319,7 +333,7 @@ class ListProductsResponse(proto.Message):
     method.
 
     Attributes:
-        products (Sequence[google.cloud.retail_v2.types.Product]):
+        products (MutableSequence[google.cloud.retail_v2.types.Product]):
             The [Product][google.cloud.retail.v2.Product]s.
         next_page_token (str):
             A token that can be sent as
@@ -332,19 +346,21 @@ class ListProductsResponse(proto.Message):
     def raw_page(self):
         return self
 
-    products = proto.RepeatedField(
+    products: MutableSequence[gcr_product.Product] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=gcr_product.Product,
     )
-    next_page_token = proto.Field(
+    next_page_token: str = proto.Field(
         proto.STRING,
         number=2,
     )
 
 
 class SetInventoryRequest(proto.Message):
-    r"""Request message for [SetInventory][] method.
+    r"""Request message for
+    [ProductService.SetInventory][google.cloud.retail.v2.ProductService.SetInventory]
+    method.
 
     Attributes:
         inventory (google.cloud.retail_v2.types.Product):
@@ -358,8 +374,9 @@ class SetInventoryRequest(proto.Message):
                The updated inventory fields must be specified in
                [SetInventoryRequest.set_mask][google.cloud.retail.v2.SetInventoryRequest.set_mask].
 
-            If [SetInventoryRequest.inventory.name][] is empty or
-            invalid, an INVALID_ARGUMENT error is returned.
+            If
+            [SetInventoryRequest.inventory.name][google.cloud.retail.v2.Product.name]
+            is empty or invalid, an INVALID_ARGUMENT error is returned.
 
             If the caller does not have permission to update the
             [Product][google.cloud.retail.v2.Product] named in
@@ -385,7 +402,7 @@ class SetInventoryRequest(proto.Message):
                [SetInventoryRequest.set_mask][google.cloud.retail.v2.SetInventoryRequest.set_mask]
             -  Specifies only the desired fulfillment types and
                corresponding place IDs to update in
-               [SetInventoryRequest.inventory.fulfillment_info][]
+               [SetInventoryRequest.inventory.fulfillment_info][google.cloud.retail.v2.Product.fulfillment_info]
 
             The caller can clear all place IDs from a subset of
             fulfillment types in the following ways:
@@ -393,10 +410,10 @@ class SetInventoryRequest(proto.Message):
             -  Adds "fulfillment_info" in
                [SetInventoryRequest.set_mask][google.cloud.retail.v2.SetInventoryRequest.set_mask]
             -  Specifies only the desired fulfillment types to clear in
-               [SetInventoryRequest.inventory.fulfillment_info][]
+               [SetInventoryRequest.inventory.fulfillment_info][google.cloud.retail.v2.Product.fulfillment_info]
             -  Checks that only the desired fulfillment info types have
                empty
-               [SetInventoryRequest.inventory.fulfillment_info.place_ids][]
+               [SetInventoryRequest.inventory.fulfillment_info.place_ids][google.cloud.retail.v2.FulfillmentInfo.place_ids]
 
             The last update time is recorded for the following inventory
             fields:
@@ -407,8 +424,9 @@ class SetInventoryRequest(proto.Message):
             -  [Product.fulfillment_info][google.cloud.retail.v2.Product.fulfillment_info]
 
             If a full overwrite of inventory information while ignoring
-            timestamps is needed, [UpdateProduct][] should be invoked
-            instead.
+            timestamps is needed,
+            [ProductService.UpdateProduct][google.cloud.retail.v2.ProductService.UpdateProduct]
+            should be invoked instead.
         set_mask (google.protobuf.field_mask_pb2.FieldMask):
             Indicates which inventory fields in the provided
             [Product][google.cloud.retail.v2.Product] to update.
@@ -434,22 +452,22 @@ class SetInventoryRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
-    inventory = proto.Field(
+    inventory: gcr_product.Product = proto.Field(
         proto.MESSAGE,
         number=1,
         message=gcr_product.Product,
     )
-    set_mask = proto.Field(
+    set_mask: field_mask_pb2.FieldMask = proto.Field(
         proto.MESSAGE,
         number=2,
         message=field_mask_pb2.FieldMask,
     )
-    set_time = proto.Field(
+    set_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=3,
         message=timestamp_pb2.Timestamp,
     )
-    allow_missing = proto.Field(
+    allow_missing: bool = proto.Field(
         proto.BOOL,
         number=4,
     )
@@ -458,21 +476,26 @@ class SetInventoryRequest(proto.Message):
 class SetInventoryMetadata(proto.Message):
     r"""Metadata related to the progress of the SetInventory operation.
     Currently empty because there is no meaningful metadata populated
-    from the [SetInventory][] method.
+    from the
+    [ProductService.SetInventory][google.cloud.retail.v2.ProductService.SetInventory]
+    method.
 
     """
 
 
 class SetInventoryResponse(proto.Message):
     r"""Response of the SetInventoryRequest. Currently empty because there
-    is no meaningful response populated from the [SetInventory][]
+    is no meaningful response populated from the
+    [ProductService.SetInventory][google.cloud.retail.v2.ProductService.SetInventory]
     method.
 
     """
 
 
 class AddFulfillmentPlacesRequest(proto.Message):
-    r"""Request message for [AddFulfillmentPlaces][] method.
+    r"""Request message for
+    [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces]
+    method.
 
     Attributes:
         product (str):
@@ -505,8 +528,8 @@ class AddFulfillmentPlacesRequest(proto.Message):
             an INVALID_ARGUMENT error is returned.
 
             This field directly corresponds to
-            [Product.fulfillment_info.type][].
-        place_ids (Sequence[str]):
+            [Product.fulfillment_info.type][google.cloud.retail.v2.FulfillmentInfo.type].
+        place_ids (MutableSequence[str]):
             Required. The IDs for this
             [type][google.cloud.retail.v2.AddFulfillmentPlacesRequest.type],
             such as the store IDs for "pickup-in-store" or the region
@@ -538,24 +561,24 @@ class AddFulfillmentPlacesRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
-    product = proto.Field(
+    product: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    type_ = proto.Field(
+    type_: str = proto.Field(
         proto.STRING,
         number=2,
     )
-    place_ids = proto.RepeatedField(
+    place_ids: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=3,
     )
-    add_time = proto.Field(
+    add_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=4,
         message=timestamp_pb2.Timestamp,
     )
-    allow_missing = proto.Field(
+    allow_missing: bool = proto.Field(
         proto.BOOL,
         number=5,
     )
@@ -564,7 +587,9 @@ class AddFulfillmentPlacesRequest(proto.Message):
 class AddFulfillmentPlacesMetadata(proto.Message):
     r"""Metadata related to the progress of the AddFulfillmentPlaces
     operation. Currently empty because there is no meaningful metadata
-    populated from the [AddFulfillmentPlaces][] method.
+    populated from the
+    [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces]
+    method.
 
     """
 
@@ -572,13 +597,16 @@ class AddFulfillmentPlacesMetadata(proto.Message):
 class AddFulfillmentPlacesResponse(proto.Message):
     r"""Response of the AddFulfillmentPlacesRequest. Currently empty because
     there is no meaningful response populated from the
-    [AddFulfillmentPlaces][] method.
+    [ProductService.AddFulfillmentPlaces][google.cloud.retail.v2.ProductService.AddFulfillmentPlaces]
+    method.
 
     """
 
 
 class AddLocalInventoriesRequest(proto.Message):
-    r"""Request message for [AddLocalInventories][] method.
+    r"""Request message for
+    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
+    method.
 
     Attributes:
         product (str):
@@ -590,7 +618,7 @@ class AddLocalInventoriesRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product], regardless of
             whether or not it exists, a PERMISSION_DENIED error is
             returned.
-        local_inventories (Sequence[google.cloud.retail_v2.types.LocalInventory]):
+        local_inventories (MutableSequence[google.cloud.retail_v2.types.LocalInventory]):
             Required. A list of inventory information at
             difference places. Each place is identified by
             its place ID. At most 3000 inventories are
@@ -629,26 +657,26 @@ class AddLocalInventoriesRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
-    product = proto.Field(
+    product: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    local_inventories = proto.RepeatedField(
+    local_inventories: MutableSequence[common.LocalInventory] = proto.RepeatedField(
         proto.MESSAGE,
         number=2,
         message=common.LocalInventory,
     )
-    add_mask = proto.Field(
+    add_mask: field_mask_pb2.FieldMask = proto.Field(
         proto.MESSAGE,
         number=4,
         message=field_mask_pb2.FieldMask,
     )
-    add_time = proto.Field(
+    add_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=5,
         message=timestamp_pb2.Timestamp,
     )
-    allow_missing = proto.Field(
+    allow_missing: bool = proto.Field(
         proto.BOOL,
         number=6,
     )
@@ -657,21 +685,28 @@ class AddLocalInventoriesRequest(proto.Message):
 class AddLocalInventoriesMetadata(proto.Message):
     r"""Metadata related to the progress of the AddLocalInventories
     operation. Currently empty because there is no meaningful metadata
-    populated from the [AddLocalInventories][] method.
+    populated from the
+    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
+    method.
 
     """
 
 
 class AddLocalInventoriesResponse(proto.Message):
-    r"""Response of the [AddLocalInventories][] API. Currently empty because
-    there is no meaningful response populated from the
-    [AddLocalInventories][] method.
+    r"""Response of the
+    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
+    API. Currently empty because there is no meaningful response
+    populated from the
+    [ProductService.AddLocalInventories][google.cloud.retail.v2.ProductService.AddLocalInventories]
+    method.
 
     """
 
 
 class RemoveLocalInventoriesRequest(proto.Message):
-    r"""Request message for [RemoveLocalInventories][] method.
+    r"""Request message for
+    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
+    method.
 
     Attributes:
         product (str):
@@ -683,7 +718,7 @@ class RemoveLocalInventoriesRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product], regardless of
             whether or not it exists, a PERMISSION_DENIED error is
             returned.
-        place_ids (Sequence[str]):
+        place_ids (MutableSequence[str]):
             Required. A list of place IDs to have their
             inventory deleted. At most 3000 place IDs are
             allowed per request.
@@ -702,20 +737,20 @@ class RemoveLocalInventoriesRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
-    product = proto.Field(
+    product: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    place_ids = proto.RepeatedField(
+    place_ids: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=2,
     )
-    remove_time = proto.Field(
+    remove_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=5,
         message=timestamp_pb2.Timestamp,
     )
-    allow_missing = proto.Field(
+    allow_missing: bool = proto.Field(
         proto.BOOL,
         number=3,
     )
@@ -724,21 +759,28 @@ class RemoveLocalInventoriesRequest(proto.Message):
 class RemoveLocalInventoriesMetadata(proto.Message):
     r"""Metadata related to the progress of the RemoveLocalInventories
     operation. Currently empty because there is no meaningful metadata
-    populated from the [RemoveLocalInventories][] method.
+    populated from the
+    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
+    method.
 
     """
 
 
 class RemoveLocalInventoriesResponse(proto.Message):
-    r"""Response of the [RemoveLocalInventories][] API. Currently empty
-    because there is no meaningful response populated from the
-    [RemoveLocalInventories][] method.
+    r"""Response of the
+    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
+    API. Currently empty because there is no meaningful response
+    populated from the
+    [ProductService.RemoveLocalInventories][google.cloud.retail.v2.ProductService.RemoveLocalInventories]
+    method.
 
     """
 
 
 class RemoveFulfillmentPlacesRequest(proto.Message):
-    r"""Request message for [RemoveFulfillmentPlaces][] method.
+    r"""Request message for
+    [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces]
+    method.
 
     Attributes:
         product (str):
@@ -771,8 +813,8 @@ class RemoveFulfillmentPlacesRequest(proto.Message):
             an INVALID_ARGUMENT error is returned.
 
             This field directly corresponds to
-            [Product.fulfillment_info.type][].
-        place_ids (Sequence[str]):
+            [Product.fulfillment_info.type][google.cloud.retail.v2.FulfillmentInfo.type].
+        place_ids (MutableSequence[str]):
             Required. The IDs for this
             [type][google.cloud.retail.v2.RemoveFulfillmentPlacesRequest.type],
             such as the store IDs for "pickup-in-store" or the region
@@ -799,24 +841,24 @@ class RemoveFulfillmentPlacesRequest(proto.Message):
             [Product][google.cloud.retail.v2.Product] is not found.
     """
 
-    product = proto.Field(
+    product: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    type_ = proto.Field(
+    type_: str = proto.Field(
         proto.STRING,
         number=2,
     )
-    place_ids = proto.RepeatedField(
+    place_ids: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=3,
     )
-    remove_time = proto.Field(
+    remove_time: timestamp_pb2.Timestamp = proto.Field(
         proto.MESSAGE,
         number=4,
         message=timestamp_pb2.Timestamp,
     )
-    allow_missing = proto.Field(
+    allow_missing: bool = proto.Field(
         proto.BOOL,
         number=5,
     )
@@ -825,7 +867,9 @@ class RemoveFulfillmentPlacesRequest(proto.Message):
 class RemoveFulfillmentPlacesMetadata(proto.Message):
     r"""Metadata related to the progress of the RemoveFulfillmentPlaces
     operation. Currently empty because there is no meaningful metadata
-    populated from the [RemoveFulfillmentPlaces][] method.
+    populated from the
+    [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces]
+    method.
 
     """
 
@@ -833,7 +877,8 @@ class RemoveFulfillmentPlacesMetadata(proto.Message):
 class RemoveFulfillmentPlacesResponse(proto.Message):
     r"""Response of the RemoveFulfillmentPlacesRequest. Currently empty
     because there is no meaningful response populated from the
-    [RemoveFulfillmentPlaces][] method.
+    [ProductService.RemoveFulfillmentPlaces][google.cloud.retail.v2.ProductService.RemoveFulfillmentPlaces]
+    method.
 
     """
 
