@@ -13,8 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import proto  # type: ignore
+from typing import MutableMapping, MutableSequence
 
+import proto  # type: ignore
 
 __protobuf__ = proto.module(
     package="google.cloud.retail.v2",
@@ -41,7 +42,7 @@ class Promotion(proto.Message):
             `promotion <https://support.google.com/merchants/answer/7050148>`__.
     """
 
-    promotion_id = proto.Field(
+    promotion_id: str = proto.Field(
         proto.STRING,
         number=1,
     )
