@@ -13,13 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+from typing import MutableMapping, MutableSequence
+
+from google.protobuf import field_mask_pb2  # type: ignore
+from google.protobuf import struct_pb2  # type: ignore
 import proto  # type: ignore
 
 from google.cloud.retail_v2.types import common
 from google.cloud.retail_v2.types import product as gcr_product
-from google.protobuf import field_mask_pb2  # type: ignore
-from google.protobuf import struct_pb2  # type: ignore
-
 
 __protobuf__ = proto.module(
     package="google.cloud.retail.v2",
@@ -37,12 +38,13 @@ class SearchRequest(proto.Message):
 
     Attributes:
         placement (str):
-            Required. The resource name of the search engine placement,
-            such as
-            ``projects/*/locations/global/catalogs/default_catalog/placements/default_search``
-            This field is used to identify the serving configuration
-            name and the set of models that will be used to make the
-            search.
+            Required. The resource name of the Retail Search serving
+            config, such as
+            ``projects/*/locations/global/catalogs/default_catalog/servingConfigs/default_serving_config``
+            or the name of the legacy placement resource, such as
+            ``projects/*/locations/global/catalogs/default_catalog/placements/default_search``.
+            This field is used to identify the serving config name and
+            the set of models that will be used to make the search.
         branch (str):
             The branch resource name, such as
             ``projects/*/locations/global/catalogs/default_catalog/branches/0``.
@@ -51,6 +53,11 @@ class SearchRequest(proto.Message):
             empty, to search products under the default branch.
         query (str):
             Raw search query.
+
+            If this field is empty, the request is considered a category
+            browsing request and returned results are based on
+            [filter][google.cloud.retail.v2.SearchRequest.filter] and
+            [page_categories][google.cloud.retail.v2.SearchRequest.page_categories].
         visitor_id (str):
             Required. A unique identifier for tracking visitors. For
             example, this could be implemented with an HTTP cookie,
@@ -129,7 +136,7 @@ class SearchRequest(proto.Message):
 
             If this field is unrecognizable, an INVALID_ARGUMENT is
             returned.
-        facet_specs (Sequence[google.cloud.retail_v2.types.SearchRequest.FacetSpec]):
+        facet_specs (MutableSequence[google.cloud.retail_v2.types.SearchRequest.FacetSpec]):
             Facet specifications for faceted search. If empty, no facets
             are returned.
 
@@ -147,18 +154,20 @@ class SearchRequest(proto.Message):
             details at this `user
             guide <https://cloud.google.com/retail/docs/boosting>`__.
 
-            Notice that if both [ServingConfig.boost_control_ids][] and
-            [SearchRequest.boost_spec] are set, the boost conditions
-            from both places are evaluated. If a search request matches
-            multiple boost conditions, the final boost score is equal to
-            the sum of the boost scores from all matched boost
-            conditions.
+            Notice that if both
+            [ServingConfig.boost_control_ids][google.cloud.retail.v2.ServingConfig.boost_control_ids]
+            and
+            [SearchRequest.boost_spec][google.cloud.retail.v2.SearchRequest.boost_spec]
+            are set, the boost conditions from both places are
+            evaluated. If a search request matches multiple boost
+            conditions, the final boost score is equal to the sum of the
+            boost scores from all matched boost conditions.
         query_expansion_spec (google.cloud.retail_v2.types.SearchRequest.QueryExpansionSpec):
             The query expansion specification that specifies the
             conditions under which query expansion will occur. See more
             details at this `user
             guide <https://cloud.google.com/retail/docs/result-size#query_expansion>`__.
-        variant_rollup_keys (Sequence[str]):
+        variant_rollup_keys (MutableSequence[str]):
             The keys to fetch and rollup the matching
             [variant][google.cloud.retail.v2.Product.Type.VARIANT]
             [Product][google.cloud.retail.v2.Product]s attributes,
@@ -188,7 +197,9 @@ class SearchRequest(proto.Message):
             -  inventory(place_id,price)
             -  inventory(place_id,original_price)
             -  inventory(place_id,attributes.key), where key is any key
-               in the [Product.inventories.attributes][] map.
+               in the
+               [Product.local_inventories.attributes][google.cloud.retail.v2.LocalInventory.attributes]
+               map.
             -  attributes.key, where key is any key in the
                [Product.attributes][google.cloud.retail.v2.Product.attributes]
                map.
@@ -240,7 +251,7 @@ class SearchRequest(proto.Message):
 
             If this field is set to an invalid value other than these,
             an INVALID_ARGUMENT error is returned.
-        page_categories (Sequence[str]):
+        page_categories (MutableSequence[str]):
             The categories associated with a category page. Required for
             category navigation queries to achieve good search quality.
             The format should be the same as
@@ -248,7 +259,7 @@ class SearchRequest(proto.Message):
 
             To represent full path of category, use '>' sign to separate
             different hierarchies. If '>' is part of the category name,
-            please replace it with other character(s).
+            replace it with other character(s).
 
             Category pages include special pages such as sales or
             promotions. For instance, a special sale page may have the
@@ -260,6 +271,43 @@ class SearchRequest(proto.Message):
             product search and faceted search.
         personalization_spec (google.cloud.retail_v2.types.SearchRequest.PersonalizationSpec):
             The specification for personalization.
+
+            Notice that if both
+            [ServingConfig.personalization_spec][google.cloud.retail.v2.ServingConfig.personalization_spec]
+            and
+            [SearchRequest.personalization_spec][google.cloud.retail.v2.SearchRequest.personalization_spec]
+            are set.
+            [SearchRequest.personalization_spec][google.cloud.retail.v2.SearchRequest.personalization_spec]
+            will override
+            [ServingConfig.personalization_spec][google.cloud.retail.v2.ServingConfig.personalization_spec].
+        labels (MutableMapping[str, str]):
+            The labels applied to a resource must meet the following
+            requirements:
+
+            -  Each resource can have multiple labels, up to a maximum
+               of 64.
+            -  Each label must be a key-value pair.
+            -  Keys have a minimum length of 1 character and a maximum
+               length of 63 characters and cannot be empty. Values can
+               be empty and have a maximum length of 63 characters.
+            -  Keys and values can contain only lowercase letters,
+               numeric characters, underscores, and dashes. All
+               characters must use UTF-8 encoding, and international
+               characters are allowed.
+            -  The key portion of a label must be unique. However, you
+               can use the same key with multiple resources.
+            -  Keys must start with a lowercase letter or international
+               character.
+
+            See `Google Cloud
+            Document <https://cloud.google.com/resource-manager/docs/creating-managing-labels#requirements>`__
+            for more details.
+        spell_correction_spec (google.cloud.retail_v2.types.SearchRequest.SpellCorrectionSpec):
+            The spell correction specification that
+            specifies the mode under which spell correction
+            will take effect.
+
+            This field is a member of `oneof`_ ``_spell_correction_spec``.
     """
 
     class SearchMode(proto.Enum):
@@ -276,11 +324,11 @@ class SearchRequest(proto.Message):
                 Required. The facet key specification.
             limit (int):
                 Maximum of facet values that should be returned for this
-                facet. If unspecified, defaults to 20. The maximum allowed
+                facet. If unspecified, defaults to 50. The maximum allowed
                 value is 300. Values above 300 will be coerced to 300.
 
                 If this field is negative, an INVALID_ARGUMENT is returned.
-            excluded_filter_keys (Sequence[str]):
+            excluded_filter_keys (MutableSequence[str]):
                 List of keys to exclude when faceting.
 
                 By default,
@@ -288,17 +336,28 @@ class SearchRequest(proto.Message):
                 is not excluded from the filter unless it is listed in this
                 field.
 
-                For example, suppose there are 100 products with color facet
-                "Red" and 200 products with color facet "Blue". A query
-                containing the filter "colorFamilies:ANY("Red")" and have
-                "colorFamilies" as
-                [FacetKey.key][google.cloud.retail.v2.SearchRequest.FacetSpec.FacetKey.key]
-                will by default return the "Red" with count 100.
+                Listing a facet key in this field allows its values to
+                appear as facet results, even when they are filtered out of
+                search results. Using this field does not affect what search
+                results are returned.
 
-                If this field contains "colorFamilies", then the query
-                returns both the "Red" with count 100 and "Blue" with count
-                200, because the "colorFamilies" key is now excluded from
-                the filter.
+                For example, suppose there are 100 products with the color
+                facet "Red" and 200 products with the color facet "Blue". A
+                query containing the filter "colorFamilies:ANY("Red")" and
+                having "colorFamilies" as
+                [FacetKey.key][google.cloud.retail.v2.SearchRequest.FacetSpec.FacetKey.key]
+                would by default return only "Red" products in the search
+                results, and also return "Red" with count 100 as the only
+                color facet. Although there are also blue products
+                available, "Blue" would not be shown as an available facet
+                value.
+
+                If "colorFamilies" is listed in "excludedFilterKeys", then
+                the query returns the facet values "Red" with count 100 and
+                "Blue" with count 200, because the "colorFamilies" key is
+                now excluded from the filter. Because this field doesn't
+                affect search results, the search results are still
+                correctly filtered to return only "Red" products.
 
                 A maximum of 100 values are allowed. Otherwise, an
                 INVALID_ARGUMENT error is returned.
@@ -386,17 +445,18 @@ class SearchRequest(proto.Message):
                        -  "inventory(place_id,price)"
                        -  "inventory(place_id,original_price)"
                        -  "inventory(place_id,attributes.key)".
-                intervals (Sequence[google.cloud.retail_v2.types.Interval]):
+                intervals (MutableSequence[google.cloud.retail_v2.types.Interval]):
                     Set only if values should be bucketized into
                     intervals. Must be set for facets with numerical
                     values. Must not be set for facet with text
                     values. Maximum number of intervals is 30.
-                restricted_values (Sequence[str]):
+                restricted_values (MutableSequence[str]):
                     Only get facet for the given restricted values. For example,
                     when using "pickupInStore" as key and set restricted values
                     to ["store123", "store456"], only facets for "store123" and
-                    "store456" are returned. Only supported on textual fields
-                    and fulfillments. Maximum is 20.
+                    "store456" are returned. Only supported on predefined
+                    textual fields, custom textual attributes and fulfillments.
+                    Maximum is 20.
 
                     Must be set for the fulfillment facet keys:
 
@@ -417,7 +477,7 @@ class SearchRequest(proto.Message):
                     -  customFulfillment4
 
                     -  customFulfillment5
-                prefixes (Sequence[str]):
+                prefixes (MutableSequence[str]):
                     Only get facet values that start with the
                     given string prefix. For example, suppose
                     "categories" has three values "Women > Shoe",
@@ -426,7 +486,7 @@ class SearchRequest(proto.Message):
                     will give only "Women > Shoe" and "Women >
                     Dress". Only supported on textual fields.
                     Maximum is 10.
-                contains (Sequence[str]):
+                contains (MutableSequence[str]):
                     Only get facet values that contains the given
                     strings. For example, suppose "categories" has
                     three values "Women > Shoe", "Women > Dress" and
@@ -434,17 +494,24 @@ class SearchRequest(proto.Message):
                     "categories" facet will give only "Women > Shoe"
                     and "Men > Shoe". Only supported on textual
                     fields. Maximum is 10.
+                case_insensitive (bool):
+                    True to make facet keys case insensitive when
+                    getting faceting values with prefixes or
+                    contains; false otherwise.
                 order_by (str):
-                    The order in which [Facet.values][] are returned.
+                    The order in which
+                    [SearchResponse.Facet.values][google.cloud.retail.v2.SearchResponse.Facet.values]
+                    are returned.
 
                     Allowed values are:
 
                     -  "count desc", which means order by
-                       [Facet.FacetValue.count][] descending.
+                       [SearchResponse.Facet.values.count][google.cloud.retail.v2.SearchResponse.Facet.FacetValue.count]
+                       descending.
 
                     -  "value desc", which means order by
-                       [Facet.FacetValue.value][] descending. Only applies to
-                       textual facets.
+                       [SearchResponse.Facet.values.value][google.cloud.retail.v2.SearchResponse.Facet.FacetValue.value]
+                       descending. Only applies to textual facets.
 
                     If not set, textual values are sorted in `natural
                     order <https://en.wikipedia.org/wiki/Natural_sort_order>`__;
@@ -464,9 +531,11 @@ class SearchRequest(proto.Message):
                     [FacetKey.key][google.cloud.retail.v2.SearchRequest.FacetSpec.FacetKey.key]
                     when query is specified.
 
-                    In the response, [FacetValue.value][] will be always "1" and
-                    [FacetValue.count][] will be the number of results that
-                    matches the query.
+                    In the response,
+                    [SearchResponse.Facet.values.value][google.cloud.retail.v2.SearchResponse.Facet.FacetValue.value]
+                    will be always "1" and
+                    [SearchResponse.Facet.values.count][google.cloud.retail.v2.SearchResponse.Facet.FacetValue.count]
+                    will be the number of results that match the query.
 
                     For example, you can set a customized facet for
                     "shipToStore", where
@@ -476,52 +545,64 @@ class SearchRequest(proto.Message):
                     is "availability: ANY("IN_STOCK") AND shipToStore:
                     ANY("123")". Then the facet will count the products that are
                     both in stock and ship to store "123".
+                return_min_max (bool):
+                    Returns the min and max value for each
+                    numerical facet intervals. Ignored for textual
+                    facets.
             """
 
-            key = proto.Field(
+            key: str = proto.Field(
                 proto.STRING,
                 number=1,
             )
-            intervals = proto.RepeatedField(
+            intervals: MutableSequence[common.Interval] = proto.RepeatedField(
                 proto.MESSAGE,
                 number=2,
                 message=common.Interval,
             )
-            restricted_values = proto.RepeatedField(
+            restricted_values: MutableSequence[str] = proto.RepeatedField(
                 proto.STRING,
                 number=3,
             )
-            prefixes = proto.RepeatedField(
+            prefixes: MutableSequence[str] = proto.RepeatedField(
                 proto.STRING,
                 number=8,
             )
-            contains = proto.RepeatedField(
+            contains: MutableSequence[str] = proto.RepeatedField(
                 proto.STRING,
                 number=9,
             )
-            order_by = proto.Field(
+            case_insensitive: bool = proto.Field(
+                proto.BOOL,
+                number=10,
+            )
+            order_by: str = proto.Field(
                 proto.STRING,
                 number=4,
             )
-            query = proto.Field(
+            query: str = proto.Field(
                 proto.STRING,
                 number=5,
             )
+            return_min_max: bool = proto.Field(
+                proto.BOOL,
+                number=11,
+            )
 
-        facet_key = proto.Field(
+        facet_key: "SearchRequest.FacetSpec.FacetKey" = proto.Field(
             proto.MESSAGE,
             number=1,
             message="SearchRequest.FacetSpec.FacetKey",
         )
-        limit = proto.Field(
+        limit: int = proto.Field(
             proto.INT32,
             number=2,
         )
-        excluded_filter_keys = proto.RepeatedField(
+        excluded_filter_keys: MutableSequence[str] = proto.RepeatedField(
             proto.STRING,
             number=3,
         )
-        enable_dynamic_position = proto.Field(
+        enable_dynamic_position: bool = proto.Field(
             proto.BOOL,
             number=4,
         )
@@ -542,7 +623,7 @@ class SearchRequest(proto.Message):
             DISABLED = 1
             ENABLED = 2
 
-        mode = proto.Field(
+        mode: "SearchRequest.DynamicFacetSpec.Mode" = proto.Field(
             proto.ENUM,
             number=1,
             enum="SearchRequest.DynamicFacetSpec.Mode",
@@ -552,13 +633,13 @@ class SearchRequest(proto.Message):
         r"""Boost specification to boost certain items.
 
         Attributes:
-            condition_boost_specs (Sequence[google.cloud.retail_v2.types.SearchRequest.BoostSpec.ConditionBoostSpec]):
+            condition_boost_specs (MutableSequence[google.cloud.retail_v2.types.SearchRequest.BoostSpec.ConditionBoostSpec]):
                 Condition boost specifications. If a product
                 matches multiple conditions in the
                 specifictions, boost scores from these
                 specifications are all applied and combined in a
                 non-linear way. Maximum number of specifications
-                is 10.
+                is 20.
             skip_boost_spec_validation (bool):
                 Whether to skip boostspec validation. If this field is set
                 to true, invalid
@@ -609,21 +690,23 @@ class SearchRequest(proto.Message):
                     condition is ignored.
             """
 
-            condition = proto.Field(
+            condition: str = proto.Field(
                 proto.STRING,
                 number=1,
             )
-            boost = proto.Field(
+            boost: float = proto.Field(
                 proto.FLOAT,
                 number=2,
             )
 
-        condition_boost_specs = proto.RepeatedField(
+        condition_boost_specs: MutableSequence[
+            "SearchRequest.BoostSpec.ConditionBoostSpec"
+        ] = proto.RepeatedField(
             proto.MESSAGE,
             number=1,
             message="SearchRequest.BoostSpec.ConditionBoostSpec",
         )
-        skip_boost_spec_validation = proto.Field(
+        skip_boost_spec_validation: bool = proto.Field(
             proto.BOOL,
             number=2,
             optional=True,
@@ -653,12 +736,12 @@ class SearchRequest(proto.Message):
             DISABLED = 1
             AUTO = 3
 
-        condition = proto.Field(
+        condition: "SearchRequest.QueryExpansionSpec.Condition" = proto.Field(
             proto.ENUM,
             number=1,
             enum="SearchRequest.QueryExpansionSpec.Condition",
         )
-        pin_unexpanded_results = proto.Field(
+        pin_unexpanded_results: bool = proto.Field(
             proto.BOOL,
             number=2,
         )
@@ -678,95 +761,130 @@ class SearchRequest(proto.Message):
             AUTO = 1
             DISABLED = 2
 
-        mode = proto.Field(
+        mode: "SearchRequest.PersonalizationSpec.Mode" = proto.Field(
             proto.ENUM,
             number=1,
             enum="SearchRequest.PersonalizationSpec.Mode",
         )
 
-    placement = proto.Field(
+    class SpellCorrectionSpec(proto.Message):
+        r"""The specification for query spell correction.
+
+        Attributes:
+            mode (google.cloud.retail_v2.types.SearchRequest.SpellCorrectionSpec.Mode):
+                The mode under which spell correction should take effect to
+                replace the original search query. Default to
+                [Mode.AUTO][google.cloud.retail.v2.SearchRequest.SpellCorrectionSpec.Mode.AUTO].
+        """
+
+        class Mode(proto.Enum):
+            r"""Enum describing under which mode spell correction should
+            occur.
+            """
+            MODE_UNSPECIFIED = 0
+            SUGGESTION_ONLY = 1
+            AUTO = 2
+
+        mode: "SearchRequest.SpellCorrectionSpec.Mode" = proto.Field(
+            proto.ENUM,
+            number=1,
+            enum="SearchRequest.SpellCorrectionSpec.Mode",
+        )
+
+    placement: str = proto.Field(
         proto.STRING,
         number=1,
     )
-    branch = proto.Field(
+    branch: str = proto.Field(
         proto.STRING,
         number=2,
     )
-    query = proto.Field(
+    query: str = proto.Field(
         proto.STRING,
         number=3,
     )
-    visitor_id = proto.Field(
+    visitor_id: str = proto.Field(
         proto.STRING,
         number=4,
     )
-    user_info = proto.Field(
+    user_info: common.UserInfo = proto.Field(
         proto.MESSAGE,
         number=5,
         message=common.UserInfo,
     )
-    page_size = proto.Field(
+    page_size: int = proto.Field(
         proto.INT32,
         number=7,
     )
-    page_token = proto.Field(
+    page_token: str = proto.Field(
         proto.STRING,
         number=8,
     )
-    offset = proto.Field(
+    offset: int = proto.Field(
         proto.INT32,
         number=9,
     )
-    filter = proto.Field(
+    filter: str = proto.Field(
         proto.STRING,
         number=10,
     )
-    canonical_filter = proto.Field(
+    canonical_filter: str = proto.Field(
         proto.STRING,
         number=28,
     )
-    order_by = proto.Field(
+    order_by: str = proto.Field(
         proto.STRING,
         number=11,
     )
-    facet_specs = proto.RepeatedField(
+    facet_specs: MutableSequence[FacetSpec] = proto.RepeatedField(
         proto.MESSAGE,
         number=12,
         message=FacetSpec,
     )
-    dynamic_facet_spec = proto.Field(
+    dynamic_facet_spec: DynamicFacetSpec = proto.Field(
         proto.MESSAGE,
         number=21,
         message=DynamicFacetSpec,
     )
-    boost_spec = proto.Field(
+    boost_spec: BoostSpec = proto.Field(
         proto.MESSAGE,
         number=13,
         message=BoostSpec,
     )
-    query_expansion_spec = proto.Field(
+    query_expansion_spec: QueryExpansionSpec = proto.Field(
         proto.MESSAGE,
         number=14,
         message=QueryExpansionSpec,
     )
-    variant_rollup_keys = proto.RepeatedField(
+    variant_rollup_keys: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=17,
     )
-    page_categories = proto.RepeatedField(
+    page_categories: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=23,
     )
-    search_mode = proto.Field(
+    search_mode: SearchMode = proto.Field(
         proto.ENUM,
         number=31,
         enum=SearchMode,
     )
-    personalization_spec = proto.Field(
+    personalization_spec: PersonalizationSpec = proto.Field(
         proto.MESSAGE,
         number=32,
         message=PersonalizationSpec,
     )
+    labels: MutableMapping[str, str] = proto.MapField(
+        proto.STRING,
+        proto.STRING,
+        number=34,
+    )
+    spell_correction_spec: SpellCorrectionSpec = proto.Field(
+        proto.MESSAGE,
+        number=35,
+        optional=True,
+        message=SpellCorrectionSpec,
+    )
 
 
 class SearchResponse(proto.Message):
@@ -775,10 +893,10 @@ class SearchResponse(proto.Message):
     method.
 
     Attributes:
-        results (Sequence[google.cloud.retail_v2.types.SearchResponse.SearchResult]):
+        results (MutableSequence[google.cloud.retail_v2.types.SearchResponse.SearchResult]):
             A list of matched items. The order represents
             the ranking.
-        facets (Sequence[google.cloud.retail_v2.types.SearchResponse.Facet]):
+        facets (MutableSequence[google.cloud.retail_v2.types.SearchResponse.Facet]):
             Results of facets requested by user.
         total_size (int):
             The estimated total count of matched items irrespective of
@@ -788,8 +906,10 @@ class SearchResponse(proto.Message):
             [total_size][google.cloud.retail.v2.SearchResponse.total_size]
             that matches.
         corrected_query (str):
-            If spell correction applies, the corrected
-            query. Otherwise, empty.
+            Contains the spell corrected query, if found. If the spell
+            correction type is AUTOMATIC, then the search results are
+            based on corrected_query. Otherwise the original query is
+            used for search.
         attribution_token (str):
             A unique search token. This should be included in the
             [UserEvent][google.cloud.retail.v2.UserEvent] logs resulting
@@ -805,15 +925,15 @@ class SearchResponse(proto.Message):
             results.
         redirect_uri (str):
             The URI of a customer-defined redirect page. If redirect
-            action is triggered, no search will be performed, and only
+            action is triggered, no search is performed, and only
             [redirect_uri][google.cloud.retail.v2.SearchResponse.redirect_uri]
             and
             [attribution_token][google.cloud.retail.v2.SearchResponse.attribution_token]
-            will be set in the response.
-        applied_controls (Sequence[str]):
+            are set in the response.
+        applied_controls (MutableSequence[str]):
             The fully qualified resource name of applied
             `controls <https://cloud.google.com/retail/docs/serving-control-rules>`__.
-        invalid_condition_boost_specs (Sequence[google.cloud.retail_v2.types.SearchRequest.BoostSpec.ConditionBoostSpec]):
+        invalid_condition_boost_specs (MutableSequence[google.cloud.retail_v2.types.SearchRequest.BoostSpec.ConditionBoostSpec]):
             The invalid
             [SearchRequest.BoostSpec.condition_boost_specs][google.cloud.retail.v2.SearchRequest.BoostSpec.condition_boost_specs]
             that are not applied during serving.
@@ -847,7 +967,7 @@ class SearchResponse(proto.Message):
                 The count of matched
                 [variant][google.cloud.retail.v2.Product.Type.VARIANT]
                 [Product][google.cloud.retail.v2.Product]s.
-            matching_variant_fields (Sequence[google.cloud.retail_v2.types.SearchResponse.SearchResult.MatchingVariantFieldsEntry]):
+            matching_variant_fields (MutableMapping[str, google.protobuf.field_mask_pb2.FieldMask]):
                 If a [variant][google.cloud.retail.v2.Product.Type.VARIANT]
                 [Product][google.cloud.retail.v2.Product] matches the search
                 query, this map indicates which
@@ -862,7 +982,7 @@ class SearchResponse(proto.Message):
                 "products.color_info" indicates there is a match between
                 "sku1" [ColorInfo][google.cloud.retail.v2.ColorInfo] and the
                 query.
-            variant_rollup_values (Sequence[google.cloud.retail_v2.types.SearchResponse.SearchResult.VariantRollupValuesEntry]):
+            variant_rollup_values (MutableMapping[str, google.protobuf.struct_pb2.Value]):
                 The rollup matching
                 [variant][google.cloud.retail.v2.Product.Type.VARIANT]
                 [Product][google.cloud.retail.v2.Product] attributes. The
@@ -898,33 +1018,56 @@ class SearchResponse(proto.Message):
                 ``{key: "pickupInStore.store1" value { number_value: 10 }}``
                 means a there are 10 variants in this product are available
                 in the store "store1".
+            personal_labels (MutableSequence[str]):
+                Specifies previous events related to this product for this
+                user based on [UserEvent][google.cloud.retail.v2.UserEvent]
+                with same
+                [SearchRequest.visitor_id][google.cloud.retail.v2.SearchRequest.visitor_id]
+                or
+                [UserInfo.user_id][google.cloud.retail.v2.UserInfo.user_id].
+
+                This is set only when
+                [SearchRequest.PersonalizationSpec.mode][google.cloud.retail.v2.SearchRequest.PersonalizationSpec.mode]
+                is
+                [SearchRequest.PersonalizationSpec.Mode.AUTO][google.cloud.retail.v2.SearchRequest.PersonalizationSpec.Mode.AUTO].
+
+                Possible values:
+
+                -  ``purchased``: Indicates that this product has been
+                   purchased before.
         """
 
-        id = proto.Field(
+        id: str = proto.Field(
             proto.STRING,
             number=1,
         )
-        product = proto.Field(
+        product: gcr_product.Product = proto.Field(
             proto.MESSAGE,
             number=2,
             message=gcr_product.Product,
         )
-        matching_variant_count = proto.Field(
+        matching_variant_count: int = proto.Field(
             proto.INT32,
             number=3,
         )
-        matching_variant_fields = proto.MapField(
+        matching_variant_fields: MutableMapping[
+            str, field_mask_pb2.FieldMask
+        ] = proto.MapField(
             proto.STRING,
             proto.MESSAGE,
             number=4,
             message=field_mask_pb2.FieldMask,
         )
-        variant_rollup_values = proto.MapField(
+        variant_rollup_values: MutableMapping[str, struct_pb2.Value] = proto.MapField(
             proto.STRING,
             proto.MESSAGE,
             number=5,
             message=struct_pb2.Value,
         )
+        personal_labels: MutableSequence[str] = proto.RepeatedField(
+            proto.STRING,
+            number=7,
+        )
 
     class Facet(proto.Message):
         r"""A facet result.
@@ -933,7 +1076,7 @@ class SearchResponse(proto.Message):
             key (str):
                 The key for this facet. E.g., "colorFamilies"
                 or "price" or "attributes.attr1".
-            values (Sequence[google.cloud.retail_v2.types.SearchResponse.Facet.FacetValue]):
+            values (MutableSequence[google.cloud.retail_v2.types.SearchResponse.Facet.FacetValue]):
                 The facet values for this field.
             dynamic_facet (bool):
                 Whether the facet is dynamically generated.
@@ -962,34 +1105,56 @@ class SearchResponse(proto.Message):
                     This field is a member of `oneof`_ ``facet_value``.
                 count (int):
                     Number of items that have this facet value.
+                min_value (float):
+                    The minimum value in the
+                    [FacetValue.interval][google.cloud.retail.v2.SearchResponse.Facet.FacetValue.interval].
+                    Only supported on numerical facets and returned if
+                    [SearchRequest.FacetSpec.FacetKey.return_min_max][google.cloud.retail.v2.SearchRequest.FacetSpec.FacetKey.return_min_max]
+                    is true.
+                max_value (float):
+                    The maximum value in the
+                    [FacetValue.interval][google.cloud.retail.v2.SearchResponse.Facet.FacetValue.interval].
+                    Only supported on numerical facets and returned if
+                    [SearchRequest.FacetSpec.FacetKey.return_min_max][google.cloud.retail.v2.SearchRequest.FacetSpec.FacetKey.return_min_max]
+                    is true.
             """
 
-            value = proto.Field(
+            value: str = proto.Field(
                 proto.STRING,
                 number=1,
                 oneof="facet_value",
             )
-            interval = proto.Field(
+            interval: common.Interval = proto.Field(
                 proto.MESSAGE,
                 number=2,
                 oneof="facet_value",
                 message=common.Interval,
             )
-            count = proto.Field(
+            count: int = proto.Field(
                 proto.INT64,
                 number=3,
             )
+            min_value: float = proto.Field(
+                proto.DOUBLE,
+                number=5,
+            )
+            max_value: float = proto.Field(
+                proto.DOUBLE,
+                number=6,
+            )
 
-        key = proto.Field(
+        key: str = proto.Field(
             proto.STRING,
             number=1,
         )
-        values = proto.RepeatedField(
+        values: MutableSequence[
+            "SearchResponse.Facet.FacetValue"
+        ] = proto.RepeatedField(
             proto.MESSAGE,
             number=2,
             message="SearchResponse.Facet.FacetValue",
         )
-        dynamic_facet = proto.Field(
+        dynamic_facet: bool = proto.Field(
             proto.BOOL,
             number=3,
         )
@@ -1009,11 +1174,11 @@ class SearchResponse(proto.Message):
                 is set to true.
         """
 
-        expanded_query = proto.Field(
+        expanded_query: bool = proto.Field(
             proto.BOOL,
             number=1,
         )
-        pinned_result_count = proto.Field(
+        pinned_result_count: int = proto.Field(
             proto.INT64,
             number=2,
         )
@@ -1022,46 +1187,48 @@ class SearchResponse(proto.Message):
     def raw_page(self):
         return self
 
-    results = proto.RepeatedField(
+    results: MutableSequence[SearchResult] = proto.RepeatedField(
         proto.MESSAGE,
         number=1,
         message=SearchResult,
     )
-    facets = proto.RepeatedField(
+    facets: MutableSequence[Facet] = proto.RepeatedField(
         proto.MESSAGE,
         number=2,
         message=Facet,
     )
-    total_size = proto.Field(
+    total_size: int = proto.Field(
         proto.INT32,
         number=3,
     )
-    corrected_query = proto.Field(
+    corrected_query: str = proto.Field(
         proto.STRING,
         number=4,
     )
-    attribution_token = proto.Field(
+    attribution_token: str = proto.Field(
         proto.STRING,
         number=5,
     )
-    next_page_token = proto.Field(
+    next_page_token: str = proto.Field(
         proto.STRING,
         number=6,
     )
-    query_expansion_info = proto.Field(
+    query_expansion_info: QueryExpansionInfo = proto.Field(
         proto.MESSAGE,
         number=7,
         message=QueryExpansionInfo,
     )
-    redirect_uri = proto.Field(
+    redirect_uri: str = proto.Field(
         proto.STRING,
         number=10,
     )
-    applied_controls = proto.RepeatedField(
+    applied_controls: MutableSequence[str] = proto.RepeatedField(
         proto.STRING,
         number=12,
     )
-    invalid_condition_boost_specs = proto.RepeatedField(
+    invalid_condition_boost_specs: MutableSequence[
+        "SearchRequest.BoostSpec.ConditionBoostSpec"
+    ] = proto.RepeatedField(
         proto.MESSAGE,
         number=14,
         message="SearchRequest.BoostSpec.ConditionBoostSpec",
