@@ -14,40 +14,46 @@
 # limitations under the License.
 #
 import os
-import mock
 
-import grpc
-from grpc.experimental import aio
+# try/except added for compatibility with python < 3.8
+try:
+    from unittest import mock
+    from unittest.mock import AsyncMock  # pragma: NO COVER
+except ImportError:  # pragma: NO COVER
+    import mock
+
 import math
-import pytest
-from proto.marshal.rules.dates import DurationRule, TimestampRule
-
 
+from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
 from google.api_core import client_options
 from google.api_core import exceptions as core_exceptions
-from google.api_core import gapic_v1
-from google.api_core import grpc_helpers
-from google.api_core import grpc_helpers_async
-from google.api_core import path_template
+import google.auth
 from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
-from google.cloud.retail_v2.services.prediction_service import (
-    PredictionServiceAsyncClient,
-)
-from google.cloud.retail_v2.services.prediction_service import PredictionServiceClient
-from google.cloud.retail_v2.services.prediction_service import transports
-from google.cloud.retail_v2.types import common
-from google.cloud.retail_v2.types import prediction_service
-from google.cloud.retail_v2.types import product
-from google.cloud.retail_v2.types import promotion
-from google.cloud.retail_v2.types import user_event
 from google.oauth2 import service_account
 from google.protobuf import duration_pb2  # type: ignore
 from google.protobuf import field_mask_pb2  # type: ignore
 from google.protobuf import struct_pb2  # type: ignore
 from google.protobuf import timestamp_pb2  # type: ignore
 from google.protobuf import wrappers_pb2  # type: ignore
-import google.auth
+import grpc
+from grpc.experimental import aio
+from proto.marshal.rules import wrappers
+from proto.marshal.rules.dates import DurationRule, TimestampRule
+import pytest
+
+from google.cloud.retail_v2.services.prediction_service import (
+    PredictionServiceAsyncClient,
+    PredictionServiceClient,
+    transports,
+)
+from google.cloud.retail_v2.types import (
+    common,
+    prediction_service,
+    product,
+    promotion,
+    user_event,
+)
 
 
 def client_cert_source_callback():
@@ -96,24 +102,26 @@ def test__get_default_mtls_endpoint():
 
 
 @pytest.mark.parametrize(
-    "client_class",
+    "client_class,transport_name",
     [
-        PredictionServiceClient,
-        PredictionServiceAsyncClient,
+        (PredictionServiceClient, "grpc"),
+        (PredictionServiceAsyncClient, "grpc_asyncio"),
     ],
 )
-def test_prediction_service_client_from_service_account_info(client_class):
+def test_prediction_service_client_from_service_account_info(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_info"
     ) as factory:
         factory.return_value = creds
         info = {"valid": True}
-        client = client_class.from_service_account_info(info)
+        client = client_class.from_service_account_info(info, transport=transport_name)
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "retail.googleapis.com:443"
+        assert client.transport._host == ("retail.googleapis.com:443")
 
 
 @pytest.mark.parametrize(
@@ -142,27 +150,33 @@ def test_prediction_service_client_service_account_always_use_jwt(
 
 
 @pytest.mark.parametrize(
-    "client_class",
+    "client_class,transport_name",
     [
-        PredictionServiceClient,
-        PredictionServiceAsyncClient,
+        (PredictionServiceClient, "grpc"),
+        (PredictionServiceAsyncClient, "grpc_asyncio"),
     ],
 )
-def test_prediction_service_client_from_service_account_file(client_class):
+def test_prediction_service_client_from_service_account_file(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_file"
     ) as factory:
         factory.return_value = creds
-        client = client_class.from_service_account_file("dummy/file/path.json")
+        client = client_class.from_service_account_file(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        client = client_class.from_service_account_json("dummy/file/path.json")
+        client = client_class.from_service_account_json(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "retail.googleapis.com:443"
+        assert client.transport._host == ("retail.googleapis.com:443")
 
 
 def test_prediction_service_client_get_transport_class():
@@ -225,6 +239,7 @@ def test_prediction_service_client_client_options(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
     # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
@@ -242,6 +257,7 @@ def test_prediction_service_client_client_options(
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
 
     # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
@@ -259,6 +275,7 @@ def test_prediction_service_client_client_options(
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
 
     # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT has
@@ -288,6 +305,25 @@ def test_prediction_service_client_client_options(
             quota_project_id="octopus",
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
+        )
+    # Check the case api_endpoint is provided
+    options = client_options.ClientOptions(
+        api_audience="https://language.googleapis.com"
+    )
+    with mock.patch.object(transport_class, "__init__") as patched:
+        patched.return_value = None
+        client = client_class(client_options=options, transport=transport_name)
+        patched.assert_called_once_with(
+            credentials=None,
+            credentials_file=None,
+            host=client.DEFAULT_ENDPOINT,
+            scopes=None,
+            client_cert_source_for_mtls=None,
+            quota_project_id=None,
+            client_info=transports.base.DEFAULT_CLIENT_INFO,
+            always_use_jwt_access=True,
+            api_audience="https://language.googleapis.com",
         )
 
 
@@ -365,6 +401,7 @@ def test_prediction_service_client_mtls_env_auto(
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
 
     # Check the case ADC client cert is provided. Whether client cert is used depends on
@@ -399,6 +436,7 @@ def test_prediction_service_client_mtls_env_auto(
                         quota_project_id=None,
                         client_info=transports.base.DEFAULT_CLIENT_INFO,
                         always_use_jwt_access=True,
+                        api_audience=None,
                     )
 
     # Check the case client_cert_source and ADC client cert are not provided.
@@ -421,6 +459,7 @@ def test_prediction_service_client_mtls_env_auto(
                     quota_project_id=None,
                     client_info=transports.base.DEFAULT_CLIENT_INFO,
                     always_use_jwt_access=True,
+                    api_audience=None,
                 )
 
 
@@ -535,6 +574,7 @@ def test_prediction_service_client_client_options_scopes(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
 
@@ -573,6 +613,7 @@ def test_prediction_service_client_client_options_credentials_file(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
 
@@ -593,6 +634,7 @@ def test_prediction_service_client_client_options_from_dict():
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
 
@@ -631,6 +673,7 @@ def test_prediction_service_client_create_channel_credentials_file(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
     # test that the credentials from file are saved and used as the credentials.
@@ -768,7 +811,7 @@ def test_predict_field_headers():
     # a field header. Set these to a non-empty value.
     request = prediction_service.PredictRequest()
 
-    request.placement = "placement/value"
+    request.placement = "placement_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(type(client.transport.predict), "__call__") as call:
@@ -784,7 +827,7 @@ def test_predict_field_headers():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "placement=placement/value",
+        "placement=placement_value",
     ) in kw["metadata"]
 
 
@@ -798,7 +841,7 @@ async def test_predict_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = prediction_service.PredictRequest()
 
-    request.placement = "placement/value"
+    request.placement = "placement_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(type(client.transport.predict), "__call__") as call:
@@ -816,7 +859,7 @@ async def test_predict_field_headers_async():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "placement=placement/value",
+        "placement=placement_value",
     ) in kw["metadata"]
 
 
@@ -911,6 +954,19 @@ def test_transport_adc(transport_class):
         adc.assert_called_once()
 
 
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+    ],
+)
+def test_transport_kind(transport_name):
+    transport = PredictionServiceClient.get_transport_class(transport_name)(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    assert transport.kind == transport_name
+
+
 def test_transport_grpc_default():
     # A client should use the gRPC transport by default.
     client = PredictionServiceClient(
@@ -951,6 +1007,14 @@ def test_prediction_service_base_transport():
     with pytest.raises(NotImplementedError):
         transport.close()
 
+    # Catch all for all remaining methods and properties
+    remainder = [
+        "kind",
+    ]
+    for r in remainder:
+        with pytest.raises(NotImplementedError):
+            getattr(transport, r)()
+
 
 def test_prediction_service_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
@@ -1016,6 +1080,28 @@ def test_prediction_service_transport_auth_adc(transport_class):
         )
 
 
+@pytest.mark.parametrize(
+    "transport_class",
+    [
+        transports.PredictionServiceGrpcTransport,
+        transports.PredictionServiceGrpcAsyncIOTransport,
+    ],
+)
+def test_prediction_service_transport_auth_gdch_credentials(transport_class):
+    host = "https://language.com"
+    api_audience_tests = [None, "https://language2.com"]
+    api_audience_expect = [host, "https://language2.com"]
+    for t, e in zip(api_audience_tests, api_audience_expect):
+        with mock.patch.object(google.auth, "default", autospec=True) as adc:
+            gdch_mock = mock.MagicMock()
+            type(gdch_mock).with_gdch_audience = mock.PropertyMock(
+                return_value=gdch_mock
+            )
+            adc.return_value = (gdch_mock, None)
+            transport_class(host=host, api_audience=t)
+            gdch_mock.with_gdch_audience.assert_called_once_with(e)
+
+
 @pytest.mark.parametrize(
     "transport_class,grpc_helpers",
     [
@@ -1096,24 +1182,40 @@ def test_prediction_service_grpc_transport_client_cert_source_for_mtls(transport
             )
 
 
-def test_prediction_service_host_no_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_prediction_service_host_no_port(transport_name):
     client = PredictionServiceClient(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="retail.googleapis.com"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "retail.googleapis.com:443"
+    assert client.transport._host == ("retail.googleapis.com:443")
 
 
-def test_prediction_service_host_with_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_prediction_service_host_with_port(transport_name):
     client = PredictionServiceClient(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="retail.googleapis.com:8000"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "retail.googleapis.com:8000"
+    assert client.transport._host == ("retail.googleapis.com:8000")
 
 
 def test_prediction_service_grpc_transport_channel():
@@ -1479,4 +1581,5 @@ def test_api_key_credentials(client_class, transport_class):
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
