@@ -14,40 +14,32 @@
 # limitations under the License.
 #
 import os
-import mock
 
-import grpc
-from grpc.experimental import aio
+# try/except added for compatibility with python < 3.8
+try:
+    from unittest import mock
+    from unittest.mock import AsyncMock  # pragma: NO COVER
+except ImportError:  # pragma: NO COVER
+    import mock
+
 import math
-import pytest
-from proto.marshal.rules.dates import DurationRule, TimestampRule
-
 
 from google.api import httpbody_pb2  # type: ignore
+from google.api_core import (
+    future,
+    gapic_v1,
+    grpc_helpers,
+    grpc_helpers_async,
+    operation,
+    operations_v1,
+    path_template,
+)
 from google.api_core import client_options
 from google.api_core import exceptions as core_exceptions
-from google.api_core import future
-from google.api_core import gapic_v1
-from google.api_core import grpc_helpers
-from google.api_core import grpc_helpers_async
-from google.api_core import operation
 from google.api_core import operation_async  # type: ignore
-from google.api_core import operations_v1
-from google.api_core import path_template
+import google.auth
 from google.auth import credentials as ga_credentials
 from google.auth.exceptions import MutualTLSChannelError
-from google.cloud.retail_v2.services.user_event_service import (
-    UserEventServiceAsyncClient,
-)
-from google.cloud.retail_v2.services.user_event_service import UserEventServiceClient
-from google.cloud.retail_v2.services.user_event_service import transports
-from google.cloud.retail_v2.types import common
-from google.cloud.retail_v2.types import import_config
-from google.cloud.retail_v2.types import product
-from google.cloud.retail_v2.types import promotion
-from google.cloud.retail_v2.types import purge_config
-from google.cloud.retail_v2.types import user_event
-from google.cloud.retail_v2.types import user_event_service
 from google.longrunning import operations_pb2
 from google.oauth2 import service_account
 from google.protobuf import any_pb2  # type: ignore
@@ -56,7 +48,26 @@ from google.protobuf import field_mask_pb2  # type: ignore
 from google.protobuf import timestamp_pb2  # type: ignore
 from google.protobuf import wrappers_pb2  # type: ignore
 from google.type import date_pb2  # type: ignore
-import google.auth
+import grpc
+from grpc.experimental import aio
+from proto.marshal.rules import wrappers
+from proto.marshal.rules.dates import DurationRule, TimestampRule
+import pytest
+
+from google.cloud.retail_v2.services.user_event_service import (
+    UserEventServiceAsyncClient,
+    UserEventServiceClient,
+    transports,
+)
+from google.cloud.retail_v2.types import (
+    common,
+    import_config,
+    product,
+    promotion,
+    purge_config,
+    user_event,
+    user_event_service,
+)
 
 
 def client_cert_source_callback():
@@ -105,24 +116,26 @@ def test__get_default_mtls_endpoint():
 
 
 @pytest.mark.parametrize(
-    "client_class",
+    "client_class,transport_name",
     [
-        UserEventServiceClient,
-        UserEventServiceAsyncClient,
+        (UserEventServiceClient, "grpc"),
+        (UserEventServiceAsyncClient, "grpc_asyncio"),
     ],
 )
-def test_user_event_service_client_from_service_account_info(client_class):
+def test_user_event_service_client_from_service_account_info(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_info"
     ) as factory:
         factory.return_value = creds
         info = {"valid": True}
-        client = client_class.from_service_account_info(info)
+        client = client_class.from_service_account_info(info, transport=transport_name)
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "retail.googleapis.com:443"
+        assert client.transport._host == ("retail.googleapis.com:443")
 
 
 @pytest.mark.parametrize(
@@ -151,27 +164,33 @@ def test_user_event_service_client_service_account_always_use_jwt(
 
 
 @pytest.mark.parametrize(
-    "client_class",
+    "client_class,transport_name",
     [
-        UserEventServiceClient,
-        UserEventServiceAsyncClient,
+        (UserEventServiceClient, "grpc"),
+        (UserEventServiceAsyncClient, "grpc_asyncio"),
     ],
 )
-def test_user_event_service_client_from_service_account_file(client_class):
+def test_user_event_service_client_from_service_account_file(
+    client_class, transport_name
+):
     creds = ga_credentials.AnonymousCredentials()
     with mock.patch.object(
         service_account.Credentials, "from_service_account_file"
     ) as factory:
         factory.return_value = creds
-        client = client_class.from_service_account_file("dummy/file/path.json")
+        client = client_class.from_service_account_file(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        client = client_class.from_service_account_json("dummy/file/path.json")
+        client = client_class.from_service_account_json(
+            "dummy/file/path.json", transport=transport_name
+        )
         assert client.transport._credentials == creds
         assert isinstance(client, client_class)
 
-        assert client.transport._host == "retail.googleapis.com:443"
+        assert client.transport._host == ("retail.googleapis.com:443")
 
 
 def test_user_event_service_client_get_transport_class():
@@ -234,6 +253,7 @@ def test_user_event_service_client_client_options(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
     # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
@@ -251,6 +271,7 @@ def test_user_event_service_client_client_options(
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
 
     # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
@@ -268,6 +289,7 @@ def test_user_event_service_client_client_options(
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
 
     # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT has
@@ -297,6 +319,25 @@ def test_user_event_service_client_client_options(
             quota_project_id="octopus",
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
+        )
+    # Check the case api_endpoint is provided
+    options = client_options.ClientOptions(
+        api_audience="https://language.googleapis.com"
+    )
+    with mock.patch.object(transport_class, "__init__") as patched:
+        patched.return_value = None
+        client = client_class(client_options=options, transport=transport_name)
+        patched.assert_called_once_with(
+            credentials=None,
+            credentials_file=None,
+            host=client.DEFAULT_ENDPOINT,
+            scopes=None,
+            client_cert_source_for_mtls=None,
+            quota_project_id=None,
+            client_info=transports.base.DEFAULT_CLIENT_INFO,
+            always_use_jwt_access=True,
+            api_audience="https://language.googleapis.com",
         )
 
 
@@ -374,6 +415,7 @@ def test_user_event_service_client_mtls_env_auto(
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
 
     # Check the case ADC client cert is provided. Whether client cert is used depends on
@@ -408,6 +450,7 @@ def test_user_event_service_client_mtls_env_auto(
                         quota_project_id=None,
                         client_info=transports.base.DEFAULT_CLIENT_INFO,
                         always_use_jwt_access=True,
+                        api_audience=None,
                     )
 
     # Check the case client_cert_source and ADC client cert are not provided.
@@ -430,6 +473,7 @@ def test_user_event_service_client_mtls_env_auto(
                     quota_project_id=None,
                     client_info=transports.base.DEFAULT_CLIENT_INFO,
                     always_use_jwt_access=True,
+                    api_audience=None,
                 )
 
 
@@ -544,6 +588,7 @@ def test_user_event_service_client_client_options_scopes(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
 
@@ -582,6 +627,7 @@ def test_user_event_service_client_client_options_credentials_file(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
 
@@ -602,6 +648,7 @@ def test_user_event_service_client_client_options_from_dict():
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
 
@@ -640,6 +687,7 @@ def test_user_event_service_client_create_channel_credentials_file(
             quota_project_id=None,
             client_info=transports.base.DEFAULT_CLIENT_INFO,
             always_use_jwt_access=True,
+            api_audience=None,
         )
 
     # test that the credentials from file are saved and used as the credentials.
@@ -822,7 +870,7 @@ def test_write_user_event_field_headers():
     # a field header. Set these to a non-empty value.
     request = user_event_service.WriteUserEventRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(type(client.transport.write_user_event), "__call__") as call:
@@ -838,7 +886,7 @@ def test_write_user_event_field_headers():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -852,7 +900,7 @@ async def test_write_user_event_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = user_event_service.WriteUserEventRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(type(client.transport.write_user_event), "__call__") as call:
@@ -870,7 +918,7 @@ async def test_write_user_event_field_headers_async():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -983,7 +1031,7 @@ def test_collect_user_event_field_headers():
     # a field header. Set these to a non-empty value.
     request = user_event_service.CollectUserEventRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1001,7 +1049,7 @@ def test_collect_user_event_field_headers():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1015,7 +1063,7 @@ async def test_collect_user_event_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = user_event_service.CollectUserEventRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1035,7 +1083,7 @@ async def test_collect_user_event_field_headers_async():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1137,7 +1185,7 @@ def test_purge_user_events_field_headers():
     # a field header. Set these to a non-empty value.
     request = purge_config.PurgeUserEventsRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1155,7 +1203,7 @@ def test_purge_user_events_field_headers():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1169,7 +1217,7 @@ async def test_purge_user_events_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = purge_config.PurgeUserEventsRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1189,7 +1237,7 @@ async def test_purge_user_events_field_headers_async():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1291,7 +1339,7 @@ def test_import_user_events_field_headers():
     # a field header. Set these to a non-empty value.
     request = import_config.ImportUserEventsRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1309,7 +1357,7 @@ def test_import_user_events_field_headers():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1323,7 +1371,7 @@ async def test_import_user_events_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = import_config.ImportUserEventsRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1343,7 +1391,7 @@ async def test_import_user_events_field_headers_async():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1446,7 +1494,7 @@ def test_rejoin_user_events_field_headers():
     # a field header. Set these to a non-empty value.
     request = user_event_service.RejoinUserEventsRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1464,7 +1512,7 @@ def test_rejoin_user_events_field_headers():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1478,7 +1526,7 @@ async def test_rejoin_user_events_field_headers_async():
     # a field header. Set these to a non-empty value.
     request = user_event_service.RejoinUserEventsRequest()
 
-    request.parent = "parent/value"
+    request.parent = "parent_value"
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1498,7 +1546,7 @@ async def test_rejoin_user_events_field_headers_async():
     _, _, kw = call.mock_calls[0]
     assert (
         "x-goog-request-params",
-        "parent=parent/value",
+        "parent=parent_value",
     ) in kw["metadata"]
 
 
@@ -1593,6 +1641,19 @@ def test_transport_adc(transport_class):
         adc.assert_called_once()
 
 
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+    ],
+)
+def test_transport_kind(transport_name):
+    transport = UserEventServiceClient.get_transport_class(transport_name)(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    assert transport.kind == transport_name
+
+
 def test_transport_grpc_default():
     # A client should use the gRPC transport by default.
     client = UserEventServiceClient(
@@ -1644,6 +1705,14 @@ def test_user_event_service_base_transport():
     with pytest.raises(NotImplementedError):
         transport.operations_client
 
+    # Catch all for all remaining methods and properties
+    remainder = [
+        "kind",
+    ]
+    for r in remainder:
+        with pytest.raises(NotImplementedError):
+            getattr(transport, r)()
+
 
 def test_user_event_service_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
@@ -1709,6 +1778,28 @@ def test_user_event_service_transport_auth_adc(transport_class):
         )
 
 
+@pytest.mark.parametrize(
+    "transport_class",
+    [
+        transports.UserEventServiceGrpcTransport,
+        transports.UserEventServiceGrpcAsyncIOTransport,
+    ],
+)
+def test_user_event_service_transport_auth_gdch_credentials(transport_class):
+    host = "https://language.com"
+    api_audience_tests = [None, "https://language2.com"]
+    api_audience_expect = [host, "https://language2.com"]
+    for t, e in zip(api_audience_tests, api_audience_expect):
+        with mock.patch.object(google.auth, "default", autospec=True) as adc:
+            gdch_mock = mock.MagicMock()
+            type(gdch_mock).with_gdch_audience = mock.PropertyMock(
+                return_value=gdch_mock
+            )
+            adc.return_value = (gdch_mock, None)
+            transport_class(host=host, api_audience=t)
+            gdch_mock.with_gdch_audience.assert_called_once_with(e)
+
+
 @pytest.mark.parametrize(
     "transport_class,grpc_helpers",
     [
@@ -1789,24 +1880,40 @@ def test_user_event_service_grpc_transport_client_cert_source_for_mtls(transport
             )
 
 
-def test_user_event_service_host_no_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_user_event_service_host_no_port(transport_name):
     client = UserEventServiceClient(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="retail.googleapis.com"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "retail.googleapis.com:443"
+    assert client.transport._host == ("retail.googleapis.com:443")
 
 
-def test_user_event_service_host_with_port():
+@pytest.mark.parametrize(
+    "transport_name",
+    [
+        "grpc",
+        "grpc_asyncio",
+    ],
+)
+def test_user_event_service_host_with_port(transport_name):
     client = UserEventServiceClient(
         credentials=ga_credentials.AnonymousCredentials(),
         client_options=client_options.ClientOptions(
             api_endpoint="retail.googleapis.com:8000"
         ),
+        transport=transport_name,
     )
-    assert client.transport._host == "retail.googleapis.com:8000"
+    assert client.transport._host == ("retail.googleapis.com:8000")
 
 
 def test_user_event_service_grpc_transport_channel():
@@ -2229,4 +2336,5 @@ def test_api_key_credentials(client_class, transport_class):
                 quota_project_id=None,
                 client_info=transports.base.DEFAULT_CLIENT_INFO,
                 always_use_jwt_access=True,
+                api_audience=None,
             )
